; NIBL Basic on a PIC16F877
;
; (c) 2017 K Orton All rights reserved
;
; This version for use with my PIC terminal emulator
;
; Port A carries control signals
; Port B is the data bus
; Port C is the low address bus
; Port D is the high address bus (except RD7)
;
; RS232 port on Port E and Port D bit 7

; PIC constants
W	EQU	0
F	EQU	1
INDF	EQU	H'0000'
TMR0	EQU	H'0001'
PCL	EQU	H'0002'
STATUS	EQU	H'0003'
FSR	EQU	H'0004'
PORTA	EQU	H'0005'
PORTB	EQU	H'0006'
PORTC	EQU	H'0007'
PORTD	EQU	H'0008'
PORTE	EQU	H'0009'
PCLATH	EQU	H'000A'
PIR1	EQU	H'000C'
TMR1L	EQU	H'000E'
TMR1H	EQU	H'000F'
T1CON	EQU	H'0010'
TRISA	EQU	H'0085'
TRISB	EQU	H'0086'
TRISC	EQU	H'0087'
TRISD	EQU	H'0088'
TRISE	EQU	H'0089'
ADCON1	EQU	H'009F'
C	EQU	0
DC	EQU	1
Z	EQU	2
RP0	EQU	5
IRP	EQU	7
CFG	EQU	H'3D32'

	__CONFIG	CFG

; Program constants
SIBUF	EQU	H'20'
FCS1	EQU	H'6A'
FCS2	EQU	H'6B'
IOBUF	EQU	H'6C'
IOCNT	EQU	H'6D'
SIPTR	EQU	H'6E'
SICNT	EQU	H'6F'
ACC	EQU	H'70'
EXT	EQU	H'71'
STAT	EQU	H'72'
P0HI	EQU	H'73'
P0LO	EQU	H'74'
P1HI	EQU	H'75'
P1LO	EQU	H'76'
P2HI	EQU	H'77'
P2LO	EQU	H'78'
P3HI	EQU	H'79'
P3LO	EQU	H'7A'
OPRND	EQU	H'7B'
EAHI	EQU	H'7C'
EALO	EQU	H'7D'
DATUM	EQU	H'7E'
TMP	EQU	H'7F'
IE	EQU	3
OV	EQU	6
CYL	EQU	7
NRD	EQU	0
NWR	EQU	1
NRAMCS	EQU	2
NIOSEL	EQU	3
NACK	EQU	4
NBTTN	EQU	5
SERCTS	EQU	7
SERRTS	EQU	0
SERRXD	EQU	1
SERTXD	EQU	2
SPOEN	EQU	0
CTSEN	EQU	1
SPIEN	EQU	2
NPADCR	EQU	3
SIMON	EQU	4
SPIOVF	EQU	5
CRCH	EQU	H'0D'
LFCH	EQU	H'0A'
CTLPCH	EQU	H'10'
CTLSCH	EQU	H'13'
PADCH	EQU	H'FF'
PADCNT	EQU	D'10'
NIPAGE	EQU	H'1012'
NIP1ST	EQU	H'1120'
T120MS	EQU	D'53036'

; Read ROM macro part 1
RDROM1	MACRO	PHI,PLO
	MOVF	PHI,W
	ADDLW	HIGH NIBL
	MOVWF	PCLATH
	MOVF	PLO,W
	MOVWF	PCL
	ENDM

; Read ROM macro part 2
; Read data returned in DST and W
RDROM2	MACRO	ROMSUB,DST,LATH
	CALL	ROMSUB
	MOVWF	DST
	MOVLW	LATH
	MOVWF	PCLATH
	MOVF	DST,W
	ENDM

; Read RAM macro
; Read data returned in W
RDRAM	MACRO	PHI,PLO
	MOVF	PLO,W
	MOVWF	PORTC
	MOVF	PHI,W
	MOVWF	PORTD
	MOVLW	B'00001010'
	MOVWF	PORTA
	NOP
	MOVF	PORTB,W
	MOVWF	TMP
	MOVLW	B'00001111'
	MOVWF	PORTA
	MOVF	TMP,W
	ENDM

; Write RAM macro
WRRAM	MACRO	PHI,PLO,SCE
	CLRF	INDF
	MOVF	SCE,W
	MOVWF	PORTB
	MOVF	PLO,W
	MOVWF	PORTC
	MOVF	PHI,W
	MOVWF	PORTD
	MOVLW	B'00001001'
	MOVWF	PORTA
	MOVLW	B'00001111'
	MOVWF	PORTA
	MOVLW	B'11111111'
	MOVWF	INDF
	ENDM

; Long jump to home page macro
HOME	MACRO	DST
	MOVLW	HIGH OPVEC
	MOVWF	PCLATH
	GOTO	DST
	ENDM

; Long jump to page 0 macro
LJMP0	MACRO	DST
	CLRF	PCLATH
	GOTO	DST
	ENDM

; Exchange byte macro
XCHNG	MACRO	A,B
	MOVF	A,W
	MOVWF	TMP
	MOVF	B,W
	MOVWF	A
	MOVF	TMP,W
	MOVWF	B
	ENDM

; Binary add macro
ADD	MACRO	SCE
	MOVF	SCE,W
	XORWF	ACC,W
	XORLW	H'80'
	MOVWF	TMP
	BTFSC	STAT,CYL
	INCFSZ	ACC,F
	BCF	STAT,CYL
	MOVF	SCE,W
	ADDWF	ACC,F
	BTFSC	STATUS,C
	BSF	STAT,CYL
	XORWF	ACC,W
	ANDWF	TMP,F
	BCF	STAT,OV
	BTFSC	TMP,7
	BSF	STAT,OV
	ENDM

; Decimal add macro
DAD	MACRO	SCE
	MOVLW	H'66'
	ADDWF	SCE,F
	BTFSC	STAT,CYL
	INCF	ACC,F
	RLF	STAT,F
	MOVF	SCE,W
	ADDWF	ACC,F
	RRF	STAT,F
	MOVLW	H'06'
	BTFSS	STATUS,DC
	SUBWF	ACC,F
	MOVLW	H'60'
	BTFSS	STAT,CYL
	SUBWF	ACC,F
	ENDM

; Increment PC macro
INCPC	MACRO
	INCF	P0HI,F
	INCFSZ	P0LO,F
	DECF	P0HI,F
	ENDM

; Get operand macro
; Operand returned in OPRND and W
GOP	MACRO	P0SUB,LATH
	LOCAL	L1
	LOCAL	L2
	INCPC
	MOVF	P0HI,W
	ANDLW	B'11110000'
	BTFSS	STATUS,Z
	GOTO	L1
	RDROM2	P0SUB,OPRND,LATH
	GOTO	L2
L1	RDRAM	P0HI,P0LO
	MOVWF	OPRND
L2
	ENDM

; Get effective address macro
; Operand expected in OPRND and W
; Effective address returned in EAHI, EALO
GEA	MACRO	PHI,PLO
	LOCAL	L1
	XORLW	H'80'
	BTFSS	STATUS,Z
	GOTO	L1
	MOVF	EXT,W
	MOVWF	OPRND
L1	MOVF	PHI,W
	MOVWF	EAHI
	BTFSC	OPRND,7
	DECF	EAHI,F
	MOVF	OPRND,W
	ADDWF	PLO,W
	MOVWF	EALO
	BTFSC	STATUS,C
	INCF	EAHI,F
	ENDM

; Effective address to PC macro
; For home page use only
; Exits directly to EXEC
GEAPC	MACRO	PHI,PLO
	LOCAL	L1
	GOP	P0PT1,HIGH OPVEC
	XORLW	H'80'

;	BTFSS	STATUS,Z			; JMP H80 bug fix
	NOP

	GOTO	L1
	MOVF	PHI,W
	MOVWF	P0HI
	BTFSC	EXT,7
	DECF	P0HI,F
	MOVF	EXT,W
	ADDWF	PLO,W
	MOVWF	P0LO
	BTFSC	STATUS,C
	INCF	P0HI,F
	GOTO	EXEC
L1	MOVF	PHI,W
	MOVWF	P0HI
	BTFSC	OPRND,7
	DECF	P0HI,F
	MOVF	OPRND,W
	ADDWF	PLO,W
	MOVWF	P0LO
	BTFSC	STATUS,C
	INCF	P0HI,F
	GOTO	EXEC
	ENDM

; Read memory macro
; Read data returned in DATUM and W
RDMEM	MACRO	PHI,PLO,PSUB,LATH
	LOCAL	L1
	LOCAL	L2
	MOVF	PHI,W
	ANDLW	B'11110000'
	BTFSS	STATUS,Z
	GOTO	L1
	RDROM2	PSUB,DATUM,LATH
	GOTO	L2
L1	RDRAM	PHI,PLO
	MOVWF	DATUM
L2
	ENDM

; Auto indexed read memory macro
; For home page use only
; Read data returned in DATUM and W
RDMEMA	MACRO	PHI,PLO,PSUB
	LOCAL	L1
	LOCAL	L2
	GOP	P0PT1,HIGH OPVEC
	GEA	PHI,PLO
	BTFSC	OPRND,7
	GOTO	L1
	RDMEM	PHI,PLO,PSUB,HIGH OPVEC
	MOVF	EAHI,W
	MOVWF	PHI
	MOVF	EALO,W
	MOVWF	PLO
	MOVF	DATUM,W
	GOTO	L2
L1	MOVF	EAHI,W
	MOVWF	PHI
	MOVF	EALO,W
	MOVWF	PLO
	RDMEM	PHI,PLO,PSUB,HIGH OPVEC
L2
	ENDM

; Auto indexed add memory macro
; For home page use only
; Exits directly to EXEC
ADMEMA	MACRO	PHI,PLO,PSUB
	LOCAL	L1
	GOP	P0PT1,HIGH OPVEC
	GEA	PHI,PLO
	BTFSC	OPRND,7
	GOTO	L1
	RDMEM	PHI,PLO,PSUB,HIGH OPVEC
	MOVF	EAHI,W
	MOVWF	PHI
	MOVF	EALO,W
	MOVWF	PLO
	ADD	DATUM
	GOTO	EXEC
L1	MOVF	EAHI,W
	MOVWF	PHI
	MOVF	EALO,W
	MOVWF	PLO
	RDMEM	PHI,PLO,PSUB,HIGH OPVEC
	ADD	DATUM
	GOTO	EXEC
	ENDM

; Write memory macro
; For page 0 use only
WRMEM	MACRO	PHI,PLO
	MOVF	PHI,W
	ANDLW	B'11110000'
	BTFSC	STATUS,Z
	RETURN
	WRRAM	PHI,PLO,ACC
	RETURN
	ENDM

; Auto indexed write memory macro
; For page 0 use only
; Exits directly to EXEC
WRMEMA	MACRO	PHI,PLO,WRSUB
	LOCAL	L1
	GOP	P0PT1X,0
	GEA	PHI,PLO
	BTFSC	OPRND,7
	GOTO	L1
	CALL	WRSUB
	MOVF	EAHI,W
	MOVWF	PHI
	MOVF	EALO,W
	MOVWF	PLO
	HOME	EXEC
L1	MOVF	EAHI,W
	MOVWF	PHI
	MOVF	EALO,W
	MOVWF	PLO
	CALL	WRSUB
	HOME	EXEC
	ENDM

; PIC reset
	ORG	0
	MOVLW	B'00001111'
	MOVWF	PORTA
	MOVLW	B'00000101'
	MOVWF	PORTE
	BSF	STATUS,RP0
	MOVLW	B'00000110'
	MOVWF	ADCON1
	MOVLW	B'00110000'
	MOVWF	TRISA
	MOVLW	B'11111111'
	MOVWF	TRISB
	CLRF	TRISC
	MOVLW	B'10000000'
	MOVWF	TRISD
	MOVLW	B'00000010'
	MOVWF	TRISE
	BCF	STATUS,RP0
	MOVLW	B'00110000'
	MOVWF	T1CON
	BCF	STATUS,IRP
	MOVLW	TRISB
	MOVWF	FSR

; Program initialisation
	MOVLW	D'255'
	MOVWF	TMP
L3	BTFSS	PORTA,NACK
	MOVWF	TMP
	DECFSZ	TMP
	GOTO	L3
	CLRF	TMR0
	CLRF	P0HI
	MOVLW	D'18'
	MOVWF	P0LO
L3A	MOVF	P0HI,W
	CALL	BANCH
	CALL	WRTERM
	INCF	P0HI
	DECFSZ	P0LO
	GOTO	L3A

; SCMP reset
RESET	CLRF	ACC
	CLRF	EXT
	CLRF	STAT
	CLRF	P0HI
	CLRF	P0LO
	CLRF	P1HI
	CLRF	P1LO
	CLRF	P2HI
	CLRF	P2LO
	CLRF	P3HI
	CLRF	P3LO
	CLRF	SICNT
	HOME	EXEC

BANCH	ADDWF	PCL
	DT	"PIC NIBL BASIC",CRCH,LFCH,CRCH,LFCH

;****************************************************************
;
;       'BIOS'
;

; NIBL character output
; Character expected in ACC
; Uses IOBUF, IOCNT
TXX	BCF	ACC,7
	MOVF	ACC,W
	CALL	WRTERM
	BTFSC	STAT,NPADCR
	GOTO	L1002
	MOVF	ACC,W
	XORLW	CRCH
	BTFSS	STATUS,Z
	GOTO	L1002
	MOVLW	PADCNT
	MOVWF	IOCNT
L1001	MOVLW	PADCH
	CALL	WRTERM
	DECFSZ	IOCNT
	GOTO	L1001
L1002	BTFSC	STAT,SPOEN
	CALL	WRSER
	HOME	EXEC

; NIBL character input
; Character returned in ACC and EXT 
RXX	BTFSS	STAT,SPIEN
	GOTO	RXTERM
	MOVF	SICNT
	BTFSC	STATUS,Z
	GOTO	L1009
	BTFSC	PORTA,NBTTN
	GOTO	L1007
	BCF	STAT,SPIEN
	CLRF	SICNT
	GOTO	RXTERM
L1007	MOVF	SIPTR,W
	MOVWF	FSR
	MOVF	INDF,W
	MOVWF	ACC
	MOVWF	EXT
	INCF	SIPTR
	DECF	SICNT
	MOVLW	TRISB
	MOVWF	FSR
	MOVF	ACC,W
	BTFSC	STAT,SIMON
	CALL	WRTNOA
L1008	HOME	EXEC
L1009	MOVLW	B'00000100'
	MOVWF	PORTE
L1010	BTFSS	PORTE,SERRXD
	GOTO	L1011
	BTFSC	PORTA,NBTTN
	GOTO	L1010
	MOVLW	B'00000101'
	MOVWF	PORTE
	BCF	STAT,SPIEN
	GOTO	RXTERM
L1011	CALL	HLFBIT
	NOP
	NOP
	BTFSC	PORTE,SERRXD
	GOTO	L1010
	CALL	FULBIT
	CALL	RDSER
	MOVWF	ACC
	MOVWF	EXT
	BTFSC	STAT,SIMON
	CALL	WRTNOA
	MOVF	ACC,W
	XORLW	CRCH
	BTFSS	STATUS,Z
	GOTO	L1008
	MOVLW	B'00000101'
	MOVWF	PORTE
	MOVLW	SIBUF
	MOVWF	SIPTR
	MOVWF	FSR
L1012	BCF	T1CON,0
	MOVLW	HIGH(T120MS)
	MOVWF	TMR1H
	MOVLW	LOW(T120MS)
	MOVWF	TMR1L
	BCF	PIR1,0
	BSF	T1CON,0
L1013	BTFSS	PORTE,SERRXD
	GOTO	L1014
	BTFSS	PIR1,0
	GOTO	L1013A
	MOVLW	TRISB
	MOVWF	FSR
	HOME	EXEC
L1013A	BTFSC	PORTA,NBTTN
	GOTO	L1013
	BCF	STAT,SPIEN
	CLRF	SICNT
	MOVLW	TRISB
	MOVWF	FSR
	GOTO	RXTERM
L1014	CALL	HLFBIT
	NOP
	NOP
	BTFSC	PORTE,SERRXD
	GOTO	L1013
	CALL	FULBIT
	CALL	RDSER
	BTFSC	SICNT,6
	BSF	STAT,SPIOVF
	BTFSC	SICNT,6
	GOTO	L1012
	MOVWF	INDF
	INCF	FSR
	INCF	SICNT
	GOTO	L1012
RXTERM	CALL	WTACK
	CALL	RDTERM
	MOVWF	ACC
L1015	CALL	RDTERM
	XORWF	ACC,W
	BTFSC	STATUS,Z
	GOTO	L1015
	CALL	RDTERM
	ANDLW	H'7F'
	MOVWF	ACC
	MOVWF	EXT
	XORLW	CTLPCH
	BTFSS	STATUS,Z
	GOTO	L1016
	MOVLW	(1<<SPOEN)
	XORWF	STAT
	GOTO	RXTERM
L1016	XORLW	(CTLPCH^CTLSCH)
	BTFSC	STATUS,Z
	GOTO	CSCALC
	BTFSC	STAT,SPOEN
	CALL	WRSER
	HOME	EXEC

; Calculate program checksum
CSCALC	MOVLW	HIGH(NIPAGE)
	MOVWF	PORTD
	MOVLW	LOW(NIPAGE)
	MOVWF	PORTC
	CALL	PEEK
	MOVWF	IOBUF
	SWAPF	IOBUF,W
	MOVWF	PORTD
	MOVLW	2
	MOVWF	PORTC
	DECFSZ	IOBUF
	GOTO	L1017
	MOVLW	HIGH(NIP1ST)
	MOVWF	PORTD
	MOVLW	LOW(NIP1ST)
	MOVWF	PORTC
L1017	CLRF	FCS1
	CLRF	FCS2
L1018	CALL	PEEK
	ADDLW	H'83'
	BTFSC	STATUS,C
	GOTO	L1021
	ADDLW	H'7D'
	CALL	ADDFCS
	CALL	PEEK
	CALL	ADDFCS
	CALL	PEEK
	CALL	ADDFCS
L1019	CALL	PEEK
	XORLW	CRCH
	BTFSC	STATUS,Z
	GOTO	L1020
	XORLW	CRCH
	CALL	ADDFCS
	GOTO	L1019
L1020	MOVLW	CRCH
	CALL	ADDFCS
	GOTO	L1018
L1021	MOVLW	CRCH
	CALL	WRTERM
	SWAPF	FCS2,W
	CALL	PRHEXD
	MOVF	FCS2,W
	CALL	PRHEXD
	SWAPF	FCS1,W
	CALL	PRHEXD
	MOVF	FCS1,W
	CALL	PRHEXD
	MOVLW	CRCH
	CALL	WRTERM
	MOVLW	'>'
	CALL	WRTERM
	GOTO	RXTERM

; Write character to serial port
; Character expected in ACC
; Uses IOCNT, IOBUF
WRSER	BTFSS	STAT,CTSEN
	GOTO	L1004
	BTFSS	PORTD,SERCTS
	GOTO	L1004
	BTFSC	PORTA,NBTTN
	GOTO	WRSER
	BCF	STAT,SPOEN
	RETURN
L1004	MOVLW	B'00000001'
	MOVWF	PORTE
	CALL	FULBIT
	MOVF	ACC,W
	MOVWF	IOBUF
	MOVLW	9
	MOVWF	IOCNT
	NOP
L1005	MOVLW	B'00000101'
	BTFSS	IOBUF,0
	MOVLW	B'00000001'
	MOVWF	PORTE
	BSF	STATUS,C
	RRF	IOBUF
	CALL	FULBIT
	DECFSZ	IOCNT
	GOTO	L1005
	RETURN

; Read character from serial port
; Character returned in W
; Uses IOCNT, IOBUF
RDSER	MOVLW	8
	MOVWF	IOCNT
L1022	BCF	STATUS,C
	RRF	IOBUF
	NOP
	BTFSC	PORTE,SERRXD
	BSF	IOBUF,7
	NOP
	CALL	FULBIT
	DECFSZ	IOCNT
	GOTO	L1022
	MOVF	IOBUF,W
	RETURN

; Delays for serial port
FULBIT	MOVLW	0	; CALL FULBIT = 4157~ (9~ short)
L1023	ADDLW	H'FF'
	BTFSS	STATUS,Z
	GOTO	L1023
	MOVLW	0
L1024	ADDLW	H'FF'
	BTFSS	STATUS,Z
	GOTO	L1024
	MOVLW	8
L1025	ADDLW	H'FF'
	BTFSS	STATUS,Z
	GOTO	L1025
	NOP
HLFBIT	MOVLW	0	; CALL HLFBIT = 2076~ (7~ short)
L1026	ADDLW	H'FF'
	BTFSS	STATUS,Z
	GOTO	L1026
	MOVLW	0
L1027	ADDLW	H'FF'
	BTFSS	STATUS,Z
	GOTO	L1027
	MOVLW	6
L1028	ADDLW	H'FF'
	BTFSS	STATUS,Z
	GOTO	L1028
	RETURN

; Read character from terminal
; Character returned in W
; Uses IOBUF
RDTERM	MOVLW	B'00000110'
	MOVWF	PORTA
	NOP
	MOVF	PORTB,W
	MOVWF	IOBUF
	MOVLW	B'00001111'
	MOVWF	PORTA
	MOVF	IOBUF,W
	RETURN

; Print a hexadecimal digit to terminal
; Digit expected in W3..W0
PRHEXD	ANDLW	H'0F'
	ADDLW	H'F6'
	BTFSC	STATUS,C
	ADDLW	('A'-D'10'-'0')
	ADDLW	(D'10'+'0')
; Drops through to...

; Write character to terminal
; Character expected in W
; Uses IOBUF
WRTERM	MOVWF	IOBUF
	CALL	WTACK
	MOVF	IOBUF,W
WRTNOA	MOVWF	PORTB
	CLRF	INDF
	CLRF	TMR0
	MOVLW	B'00000101'
	MOVWF	PORTA
	MOVLW	B'00001111'
	MOVWF	PORTA
	MOVLW	B'11111111'
	MOVWF	INDF
	RETURN

; Wait for acknowledgement from terminal
WTACK	BCF	T1CON,0
	MOVLW	HIGH(T120MS)
	MOVWF	TMR1H
	MOVLW	LOW(T120MS)
	MOVWF	TMR1L
	BCF	PIR1,0
	BSF	T1CON,0
L1029	MOVF	TMR0,W
	BTFSS	STATUS,Z
	RETURN
	BTFSS	PIR1,0
	GOTO	L1029
	RETURN

; Peek memory
; Address expected in PORTD:PORTC
; Byte returned in W
; Uses IOBUF
; Post increments PORTD:PORTC
PEEK	MOVLW	B'00001010'
	MOVWF	PORTA
	NOP
	MOVF	PORTB,W
	MOVWF	IOBUF
	MOVLW	B'00001111'
	MOVWF	PORTA
	MOVF	IOBUF,W
	INCFSZ	PORTC
	RETURN
	INCF	PORTD	
	RETURN

; Add byte to Fletcher checksum
; Byte to add expected in W
ADDFCS	ADDWF	FCS1
	BTFSC	STATUS,C
	INCF	FCS1
	INCFSZ	FCS1
	DECF	FCS1
	MOVF	FCS1,W
	ADDWF	FCS2
	BTFSC	STATUS,C
	INCF	FCS2
	INCFSZ	FCS2
	DECF	FCS2
	RETURN

; Check for break
CHKX	MOVLW	B'00100000'
	MOVWF	ACC
	BTFSS	STAT,SPIEN
	GOTO	L1029A
	BTFSS	PORTE,SERRXD
	CLRF	ACC
L1029A	BTFSC	PORTA,NBTTN
	GOTO	L1030
	CLRF	ACC
	BCF	STAT,SPIEN
	BCF	STAT,NPADCR
	CLRF	SICNT
L1030	HOME	EXEC

;****************************************************************
;
;       SC/MP emulator
;

; Opcode functions
XAEX	XCHNG	ACC,EXT
	HOME	EXEC
XPAL0X	XCHNG	ACC,P0LO
	HOME	EXEC
XPAL1X	XCHNG	ACC,P1LO
	HOME	EXEC
XPAL2X	XCHNG	ACC,P2LO
	HOME	EXEC
XPAL3X	XCHNG	ACC,P3LO
	HOME	EXEC
XPAH0X	XCHNG	ACC,P0HI
	HOME	EXEC
XPAH1X	XCHNG	ACC,P1HI
	HOME	EXEC
XPAH2X	XCHNG	ACC,P2HI
	HOME	EXEC
XPAH3X	XCHNG	ACC,P3HI
	HOME	EXEC
LD0X	CALL	RD0X
	MOVWF	ACC
	HOME	EXEC
LD1X	CALL	RD1X
	MOVWF	ACC
	HOME	EXEC
LD2X	CALL	RD2X
	MOVWF	ACC
	HOME	EXEC
LD3X	CALL	RD3X
	MOVWF	ACC
	HOME	EXEC
ADD0X	GOP	P0PT1X,0
	GEA	P0HI,P0LO
	RDMEM	EAHI,EALO,EAPT1X,0
	ADD	DATUM
	HOME	EXEC
ADD1X	GOP	P0PT1X,0
	GEA	P1HI,P1LO
	RDMEM	EAHI,EALO,EAPT1X,0
	ADD	DATUM
	HOME	EXEC
ADD2X	GOP	P0PT1X,0
	GEA	P2HI,P2LO
	RDMEM	EAHI,EALO,EAPT1X,0
	ADD	DATUM
	HOME	EXEC
ADD3X	GOP	P0PT1X,0
	GEA	P3HI,P3LO
	RDMEM	EAHI,EALO,EAPT1X,0
	ADD	DATUM
	HOME	EXEC
CAD0X	CALL	RD0X
	COMF	DATUM,F
	ADD	DATUM
	HOME	EXEC
CAD1X	CALL	RD1X
	COMF	DATUM,F
	ADD	DATUM
	HOME	EXEC
CAD2X	CALL	RD2X
	COMF	DATUM,F
	ADD	DATUM
	HOME	EXEC
CAD3X	CALL	RD3X
	COMF	DATUM,F
	ADD	DATUM
	HOME	EXEC
AND0X	CALL	RD0X
	ANDWF	ACC,F
	HOME	EXEC
AND1X	CALL	RD1X
	ANDWF	ACC,F
	HOME	EXEC
AND2X	CALL	RD2X
	ANDWF	ACC,F
	HOME	EXEC
AND3X	CALL	RD3X
	ANDWF	ACC,F
	HOME	EXEC
OR0X	CALL	RD0X
	IORWF	ACC,F
	HOME	EXEC
OR1X	CALL	RD1X
	IORWF	ACC,F
	HOME	EXEC
OR2X	CALL	RD2X
	IORWF	ACC,F
	HOME	EXEC
OR3X	CALL	RD3X
	IORWF	ACC,F
	HOME	EXEC
XOR0X	CALL	RD0X
	XORWF	ACC,F
	HOME	EXEC
XOR1X	CALL	RD1X
	XORWF	ACC,F
	HOME	EXEC
XOR2X	CALL	RD2X
	XORWF	ACC,F
	HOME	EXEC
XOR3X	CALL	RD3X
	XORWF	ACC,F
	HOME	EXEC
DAD0X	CALL	RD0X
	CALL	DADDX
	HOME	EXEC
DAD1X	CALL	RD1X
	CALL	DADDX
	HOME	EXEC
DAD2X	CALL	RD2X
	CALL	DADDX
	HOME	EXEC
DAD3X	CALL	RD3X
	CALL	DADDX
	HOME	EXEC
LDIX	CALL	GOPX
	MOVWF	ACC
	HOME	EXEC
ANIX	CALL	GOPX
	ANDWF	ACC,F
	HOME	EXEC
ORIX	CALL	GOPX
	IORWF	ACC,F
	HOME	EXEC
XRIX	CALL	GOPX
	XORWF	ACC,F
	HOME	EXEC
DAEX	DAD	EXT
	HOME	EXEC
DAIX	CALL	GOPX
	DAD	OPRND
	HOME	EXEC
CAIX	GOP	P0PT1X,0
	COMF	OPRND,F
	ADD	OPRND
	HOME	EXEC
ILD0X	CALL	RD0X
	ADDLW	1
	MOVWF	ACC
	CALL	WREAX
	HOME	EXEC
ILD1X	CALL	RD1X
	ADDLW	1
	MOVWF	ACC
	CALL	WREAX
	HOME	EXEC
ILD2X	CALL	RD2X
	ADDLW	1
	MOVWF	ACC
	CALL	WREAX
	HOME	EXEC
ILD3X	CALL	RD3X
	ADDLW	1
	MOVWF	ACC
	CALL	WREAX
	HOME	EXEC
DLD0X	CALL	RD0X
	ADDLW	H'FF'
	MOVWF	ACC
	CALL	WREAX
	HOME	EXEC
DLD1X	CALL	RD1X
	ADDLW	H'FF'
	MOVWF	ACC
	CALL	WREAX
	HOME	EXEC
DLD2X	CALL	RD2X
	ADDLW	H'FF'
	MOVWF	ACC
	CALL	WREAX
	HOME	EXEC
DLD3X	CALL	RD3X
	ADDLW	H'FF'
	MOVWF	ACC
	CALL	WREAX
	HOME	EXEC
ST0X	CALL	EA0X
	CALL	WREAX
	HOME	EXEC
ST1X	CALL	EA1X
	CALL	WREAX
	HOME	EXEC
ST2X	CALL	EA2X
	CALL	WREAX
	HOME	EXEC
ST3X	CALL	EA3X
	CALL	WREAX
	HOME	EXEC
ST1AX	WRMEMA	P1HI,P1LO,WR1X
ST2AX	WRMEMA	P2HI,P2LO,WR2X
ST3AX	WRMEMA	P3HI,P3LO,WR3X
DLYX	INCPC
	MOVLW	H'FF'
	MOVWF	ACC
	HOME	EXEC

; Page 0 opcode support
P0PT1X	RDROM1	P0HI,P0LO
EAPT1X	RDROM1	EAHI,EALO
GOPX	GOP	P0PT1X,0
	RETURN
RD0X	GOP	P0PT1X,0
	GEA	P0HI,P0LO
	RDMEM	EAHI,EALO,EAPT1X,0
	RETURN
RD1X	GOP	P0PT1X,0
	GEA	P1HI,P1LO
	RDMEM	EAHI,EALO,EAPT1X,0
	RETURN
RD2X	GOP	P0PT1X,0
	GEA	P2HI,P2LO
	RDMEM	EAHI,EALO,EAPT1X,0
	RETURN
RD3X	GOP	P0PT1X,0
	GEA	P3HI,P3LO
	RDMEM	EAHI,EALO,EAPT1X,0
	RETURN
DADDX	DAD	DATUM
	RETURN
EA0X	CALL	GOPX
	GEA	P0HI,P0LO
	RETURN
EA1X	CALL	GOPX
	GEA	P1HI,P1LO
	RETURN
EA2X	CALL	GOPX
	GEA	P2HI,P2LO
	RETURN
EA3X	CALL	GOPX
	GEA	P3HI,P3LO
	RETURN
WREAX	WRMEM	EAHI,EALO
WR1X	WRMEM	P1HI,P1LO
WR2X	WRMEM	P2HI,P2LO
WR3X	WRMEM	P3HI,P3LO

; Page 1 - main emulation kernel (home page)
	ORG	H'0800'

; Opcode vector table
OPVEC	GOTO	XHALT	; 00 HALT    8~(SCMP)
	GOTO	XAE	; 01 XAE     7~
	GOTO	CCL	; 02 CCL     5~
	GOTO	SCL	; 03 SCL     5~
	GOTO	DINT	; 04 DINT    6~
	GOTO	IEN	; 05 IEN     6~
	GOTO	CSA	; 06 CSA     5~
	GOTO	CAS	; 07 CAS     6~
	GOTO	XNOP	; 08 NOP     5~
	GOTO	ILL	; 09
	GOTO	ILL	; 0A
	GOTO	ILL	; 0B
	GOTO	ILL	; 0C
	GOTO	ILL	; 0D
	GOTO	ILL	; 0E
	GOTO	ILL	; 0F
	GOTO	ILL	; 10
	GOTO	ILL	; 11
	GOTO	ILL	; 12
	GOTO	ILL	; 13
	GOTO	ILL	; 14
	GOTO	ILL	; 15
	GOTO	ILL	; 16
	GOTO	ILL	; 17
	GOTO	ILL	; 18
	GOTO	SIO	; 19 SIO     5~
	GOTO	ILL	; 1A
	GOTO	ILL	; 1B
	GOTO	SR	; 1C SR      5~  
	GOTO	SRL	; 1D SRL     5~
	GOTO	RR	; 1E RR      5~
	GOTO	RRL	; 1F RRL     5~
	GOTO	TXCHAR	; 20 TXCHAR (pseudo instruction)
	GOTO	RXCHAR	; 21 RXCHAR	"
	GOTO	CHKBRK	; 22 CHKBRK	"
	GOTO	ILL	; 23
	GOTO	ILL	; 24
	GOTO	ILL	; 25
	GOTO	ILL	; 26
	GOTO	ILL	; 27
	GOTO	ILL	; 28
	GOTO	ILL	; 29
	GOTO	ILL	; 2A
	GOTO	ILL	; 2B
	GOTO	ILL	; 2C
	GOTO	ILL	; 2D
	GOTO	ILL	; 2E
	GOTO	ILL	; 2F
	GOTO	XPAL0	; 30 XPAL0   8~
	GOTO	XPAL1	; 31 XPAL1   8~  
	GOTO	XPAL2	; 32 XPAL2   8~
	GOTO	XPAL3	; 33 XPAL3   8~ 
	GOTO	XPAH0	; 34 XPAH0   8~
	GOTO	XPAH1	; 35 XPAH1   8~
	GOTO	XPAH2	; 36 XPAH2   8~
	GOTO	XPAH3	; 37 XPAH3   8~
	GOTO	ILL	; 38
	GOTO	ILL	; 39
	GOTO	ILL	; 3A
	GOTO	ILL	; 3B
	GOTO	XPPC0	; 3C XPPC0   7~
	GOTO	XPPC1	; 3D XPPC1   7~
	GOTO	XPPC2	; 3E XPPC2   7~
	GOTO	XPPC3	; 3F XPPC3   7~
	GOTO	LDE	; 40 LDE     6~
	GOTO	ILL	; 41
	GOTO	ILL	; 42
	GOTO	ILL	; 43
	GOTO	ILL	; 44
	GOTO	ILL	; 45
	GOTO	ILL	; 46
	GOTO	ILL	; 47
	GOTO	ILL	; 48
	GOTO	ILL	; 49
	GOTO	ILL	; 4A
	GOTO	ILL	; 4B
	GOTO	ILL	; 4C
	GOTO	ILL	; 4D
	GOTO	ILL	; 4E
	GOTO	ILL	; 4F
	GOTO	ANE	; 50 ANE     6~
	GOTO	ILL	; 51
	GOTO	ILL	; 52
	GOTO	ILL	; 53
	GOTO	ILL	; 54
	GOTO	ILL	; 55
	GOTO	ILL	; 56
	GOTO	ILL	; 57
	GOTO	ORE	; 58 ORE     6~
	GOTO	ILL	; 59
	GOTO	ILL	; 5A
	GOTO	ILL	; 5B
	GOTO	ILL	; 5C
	GOTO	ILL	; 5D
	GOTO	ILL	; 5E
	GOTO	ILL	; 5F
	GOTO	XRE	; 60 XRE     6~
	GOTO	ILL	; 61
	GOTO	ILL	; 62
	GOTO	ILL	; 63
	GOTO	ILL	; 64
	GOTO	ILL	; 65
	GOTO	ILL	; 66
	GOTO	ILL	; 67
	GOTO	DAE	; 68 DAE     11~
	GOTO	ILL	; 69
	GOTO	ILL	; 6A
	GOTO	ILL	; 6B
	GOTO	ILL	; 6C
	GOTO	ILL	; 6D
	GOTO	ILL	; 6E
	GOTO	ILL	; 6F
	GOTO	ADE	; 70 ADE     7~
	GOTO	ILL	; 71
	GOTO	ILL	; 72
	GOTO	ILL	; 73
	GOTO	ILL	; 74
	GOTO	ILL	; 75
	GOTO	ILL	; 76
	GOTO	ILL	; 77
	GOTO	CAE	; 78 CAE     8~
	GOTO	ILL	; 79
	GOTO	ILL	; 7A
	GOTO	ILL	; 7B
	GOTO	ILL	; 7C
	GOTO	ILL	; 7D
	GOTO	ILL	; 7E
	GOTO	ILL	; 7F
	GOTO	ILL	; 80
	GOTO	ILL	; 81
	GOTO	ILL	; 82
	GOTO	ILL	; 83
	GOTO	ILL	; 84
	GOTO	ILL	; 85
	GOTO	ILL	; 86
	GOTO	ILL	; 87
	GOTO	ILL	; 88
	GOTO	ILL	; 89
	GOTO	ILL	; 8A
	GOTO	ILL	; 8B
	GOTO	ILL	; 8C
	GOTO	ILL	; 8D
	GOTO	ILL	; 8E
	GOTO	DLY	; 8F DLY     13-131593~
	GOTO	JMP0	; 90 JMP0    11~
	GOTO	JMP1	; 91 JMP1    11~
	GOTO	JMP2	; 92 JMP2    11~
	GOTO	JMP3	; 93 JMP3    11~
	GOTO	JP0	; 94 JP0     9/11~
	GOTO	JP1	; 95 JP1     9/11~
	GOTO	JP2	; 96 JP2     9/11~
	GOTO	JP3	; 97 JP3     9/11~
	GOTO	JZ0	; 98 JZ0     9/11~
	GOTO	JZ1	; 99 JZ1     9/11~
	GOTO	JZ2	; 9A JZ2     9/11~
	GOTO	JZ3	; 9B JZ3     9/11~
	GOTO	JNZ0	; 9C JNZ0    9/11~
	GOTO	JNZ1	; 9D JNZ1    9/11~
	GOTO	JNZ2	; 9E JNZ2    9/11~
	GOTO	JNZ3	; 9F JNZ3    9/11~
	GOTO	ILL	; A0	
	GOTO	ILL	; A1
	GOTO	ILL	; A2
	GOTO	ILL	; A3
	GOTO	ILL	; A4
	GOTO	ILL	; A5
	GOTO	ILL	; A6
	GOTO	ILL	; A7
	GOTO	ILD0	; A8 ILD0    22~
	GOTO	ILD1	; A9 ILD1    22~
	GOTO	ILD2	; AA ILD2    22~
	GOTO	ILD3	; AB ILD3    22~
	GOTO	ILL	; AC
	GOTO	ILL	; AD
	GOTO	ILL	; AE
	GOTO	ILL	; AF
	GOTO	ILL	; B0
	GOTO	ILL	; B1
	GOTO	ILL	; B2
	GOTO	ILL	; B3
	GOTO	ILL	; B4
	GOTO	ILL	; B5
	GOTO	ILL	; B6
	GOTO	ILL	; B7
	GOTO	DLD0	; B8 DLD0    22~
	GOTO	DLD1	; B9 DLD1    22~
	GOTO	DLD2	; BA DLD2    22~
	GOTO	DLD3	; BB DLD3    22~
	GOTO	ILL	; BC
	GOTO	ILL	; BD
	GOTO	ILL	; BE
	GOTO	ILL	; BF
	GOTO	LD0	; C0 LD(0)   18~
	GOTO	LD1	; C1 LD(1)   18~
	GOTO	LD2	; C2 LD(2)   18~
	GOTO	LD3	; C3 LD(3)   18~
	GOTO	LDI	; C4 LDI     10~
	GOTO	LD1A	; C5 LD@(1)  18~
	GOTO	LD2A	; C6 LD@(2)  18~
	GOTO	LD3A	; C7 LD@(3)  18~
	GOTO	ST0	; C8 ST(0)   18~
	GOTO	ST1	; C9 ST(1)   18~
	GOTO	ST2	; CA ST(2)   18~
	GOTO	ST3	; CB ST(3)   18~
	GOTO	ILL	; CC
	GOTO	ST1A	; CD ST@(1)  18~
	GOTO	ST2A	; CE ST@(2)  18~
	GOTO	ST3A	; CF ST@(3)  18~  
	GOTO	AND0	; D0 AND(0)  18~
	GOTO	AND1	; D1 AND(1)  18~
	GOTO	AND2	; D2 AND(2)  18~
	GOTO	AND3	; D3 AND(3)  18~
	GOTO	ANI	; D4 ANI     10~
	GOTO	AND1A	; D5 AND@(1) 18~
	GOTO	AND2A	; D6 AND@(2) 18~
	GOTO	AND3A	; D7 AND@(3) 18~
	GOTO	OR0	; D8 OR(0)   18~
	GOTO	OR1	; D9 OR(1)   18~
	GOTO	OR2	; DA OR(2)   18~
	GOTO	OR3	; DB OR(3)   18~
	GOTO	ORI	; DC ORI     10~
	GOTO	OR1A	; DD OR@(1)  18~
	GOTO	OR2A	; DE OR@(2)  18~
	GOTO	OR3A	; DF OR@(3)  18~
	GOTO	XOR0	; E0 XOR(0)  18~
	GOTO	XOR1	; E1 XOR(1)  18~
	GOTO	XOR2	; E2 XOR(2)  18~
	GOTO	XOR3	; E3 XOR(3)  18~
	GOTO	XRI	; E4 XRI     10~
	GOTO	XOR1A	; E5 XOR@(1) 18~
	GOTO	XOR2A	; E6 XOR@(2) 18~
	GOTO	XOR3A	; E7 XOR@(3) 18~
	GOTO	DAD0	; E8 DAD(0)  23~
	GOTO	DAD1	; E9 DAD(1)  23~
	GOTO	DAD2	; EA DAD(2)  23~
	GOTO	DAD3	; EB DAD(3)  23~
	GOTO	DAI	; EC DAI     15~
	GOTO	DAD1A	; ED DAD@(1) 23~
	GOTO	DAD2A	; EE DAD@(2) 23~
	GOTO	DAD3A	; EF DAD@(3) 23~
	GOTO	ADD0	; F0 ADD(0)  19~
	GOTO	ADD1	; F1 ADD(1)  19~
	GOTO	ADD2	; F2 ADD(2)  19~
	GOTO	ADD3	; F3 ADD(3)  19~
	GOTO	ADI	; F4 ADI     11~
	GOTO	ADD1A	; F5 ADD@(1) 19~
	GOTO	ADD2A	; F6 ADD@(2) 19~
	GOTO	ADD3A	; F7 ADD@(3) 19~
	GOTO	CAD0	; F8 CAD(0)  20~
	GOTO	CAD1	; F9 CAD(1)  20~
	GOTO	CAD2	; FA CAD(2)  20~
	GOTO	CAD3	; FB CAD(3)  20~
	GOTO	CAI	; FC CAI     12~	
	GOTO	CAD1A	; FD CAD@(1) 20~
	GOTO	CAD2A	; FE CAD@(2) 20~
	GOTO	CAD3A	; FF CAD@(3) 20~

; Critical opcode - ADE
ADE	ADD	EXT

; Execute SCMP instruction (19)
; NB PCLATH must be set to HIGH OPVEC
XHALT
XPPC0
XNOP
EXEC	INCPC
	MOVF	P0HI,W
	ANDLW	B'11110000'
	BTFSC	STATUS,Z
	GOTO	L8
	RDRAM	P0HI,P0LO
	MOVWF	PCL
L8	RDROM2	P0PT1,TMP,HIGH OPVEC
	MOVWF	PCL

; Opcode functions
CAE	MOVF	EXT,W
	MOVWF	DATUM
	COMF	DATUM,F
	ADD	DATUM
	GOTO	EXEC
RRL	RLF	STAT,F
	RRF	ACC,F
	RRF	STAT,F
	GOTO	EXEC
LDE	MOVF	EXT,W
	MOVWF	ACC
	GOTO	EXEC
ORE	MOVF	EXT,W
	IORWF	ACC,F
	GOTO	EXEC
XRE	MOVF	EXT,W
	XORWF	ACC,F
	GOTO	EXEC
ANE	MOVF	EXT,W
	ANDWF	ACC,F
	GOTO	EXEC
SIO	RRF	EXT,F
	BCF	EXT,7
	GOTO	EXEC
SR	RRF	ACC,F
	BCF	ACC,7
	GOTO	EXEC
RR	RRF	ACC,W
	RRF	ACC,F
	GOTO	EXEC
SRL	RLF	STAT,W
	RRF	ACC,F
	GOTO	EXEC
CCL	BCF	STAT,CYL
	GOTO	EXEC
SCL	BSF	STAT,CYL
	GOTO	EXEC
DINT	BCF	STAT,IE
	GOTO	EXEC
IEN	BSF	STAT,IE
	GOTO	EXEC
CAS	MOVF	ACC,W
	MOVWF	STAT
	GOTO	EXEC
CSA	MOVF	STAT,W
	MOVWF	ACC
	GOTO	EXEC
XAE	LJMP0	XAEX
XPAL0	LJMP0	XPAL0X
XPAL1	LJMP0	XPAL1X
XPAL2	LJMP0	XPAL2X
XPAL3	LJMP0	XPAL3X
XPAH0	LJMP0	XPAH0X
XPAH1	LJMP0	XPAH1X
XPAH2	LJMP0	XPAH2X
XPAH3	LJMP0	XPAH3X
XPPC1	XCHNG	P1HI,P0HI
	XCHNG	P1LO,P0LO
	GOTO	EXEC
XPPC2	XCHNG	P2HI,P0HI
	XCHNG	P2LO,P0LO
	GOTO	EXEC
XPPC3	XCHNG	P3HI,P0HI
	XCHNG	P3LO,P0LO
	GOTO	EXEC
JP0	BTFSS	ACC,7
	GOTO	JMP0
	GOTO	NOJMP
JP1	BTFSS	ACC,7
	GOTO	JMP1
	GOTO	NOJMP
JP2	BTFSS	ACC,7
	GOTO	JMP2
	GOTO	NOJMP
JP3	BTFSS	ACC,7
	GOTO	JMP3
	GOTO	NOJMP
JZ0	MOVF	ACC,F
	BTFSC	STATUS,Z
	GOTO	JMP0
	GOTO	NOJMP
JZ1	MOVF	ACC,F
	BTFSC	STATUS,Z
	GOTO	JMP1
	GOTO	NOJMP
JZ2	MOVF	ACC,F
	BTFSC	STATUS,Z
	GOTO	JMP2
	GOTO	NOJMP
JZ3	MOVF	ACC,F
	BTFSC	STATUS,Z
	GOTO	JMP3
	GOTO	NOJMP
JNZ0	MOVF	ACC,F
	BTFSS	STATUS,Z
	GOTO	JMP0
	GOTO	NOJMP
JNZ1	MOVF	ACC,F
	BTFSS	STATUS,Z
	GOTO	JMP1
	GOTO	NOJMP
JNZ2	MOVF	ACC,F
	BTFSS	STATUS,Z
	GOTO	JMP2
	GOTO	NOJMP
JNZ3	MOVF	ACC,F
	BTFSS	STATUS,Z
	GOTO	JMP3
	GOTO	NOJMP
ILD0	LJMP0	ILD0X
ILD1	LJMP0	ILD1X
ILD2	LJMP0	ILD2X
ILD3	LJMP0	ILD3X
DLD0	LJMP0	DLD0X
DLD1	LJMP0	DLD1X
DLD2	LJMP0	DLD2X
DLD3	LJMP0	DLD3X
LD0	LJMP0	LD0X
LD1	LJMP0	LD1X
LD2	LJMP0	LD2X
LD3	LJMP0	LD3X
LD1A	CALL	RDA1
	MOVWF	ACC
	GOTO	EXEC
LD2A	CALL	RDA2
	MOVWF	ACC
	GOTO	EXEC
LD3A	CALL	RDA3
	MOVWF	ACC
	GOTO	EXEC
AND0	LJMP0	AND0X
AND1	LJMP0	AND1X
AND2	LJMP0	AND2X
AND3	LJMP0	AND3X
AND1A	CALL	RDA1
	ANDWF	ACC,F
	GOTO	EXEC
AND2A	CALL	RDA2
	ANDWF	ACC,F
	GOTO	EXEC
AND3A	CALL	RDA3
	ANDWF	ACC,F
	GOTO	EXEC
OR0	LJMP0	OR0X
OR1	LJMP0	OR1X
OR2	LJMP0	OR2X
OR3	LJMP0	OR3X
OR1A	CALL	RDA1
	IORWF	ACC,F
	GOTO	EXEC
OR2A	CALL	RDA2
	IORWF	ACC,F
	GOTO	EXEC
OR3A	CALL	RDA3
	IORWF	ACC,F
	GOTO	EXEC
XOR0	LJMP0	XOR0X
XOR1	LJMP0	XOR1X
XOR2	LJMP0	XOR2X
XOR3	LJMP0	XOR3X
XOR1A	CALL	RDA1
	XORWF	ACC,F
	GOTO	EXEC
XOR2A	CALL	RDA2
	XORWF	ACC,F
	GOTO	EXEC
XOR3A	CALL	RDA3
	XORWF	ACC,F
	GOTO	EXEC
DAD0	LJMP0	DAD0X
DAD1	LJMP0	DAD1X
DAD2	LJMP0	DAD2X
DAD3	LJMP0	DAD3X
DAD1A	CALL	RDA1
	CALL	DADD
	GOTO	EXEC
DAD2A	CALL	RDA2
	CALL	DADD
	GOTO	EXEC
DAD3A	CALL	RDA3
	CALL	DADD
	GOTO	EXEC
ADD0	LJMP0	ADD0X
ADD1	LJMP0	ADD1X
ADD2	LJMP0	ADD2X
ADD3	LJMP0	ADD3X
ADI	GOP	P0PT1,HIGH OPVEC
	ADD	OPRND
	GOTO	EXEC
ADD1A	ADMEMA	P1HI,P1LO,P1PT1
ADD2A	ADMEMA	P2HI,P2LO,P2PT1
ADD3A	ADMEMA	P3HI,P3LO,P3PT1
CAD1A	RDMEMA	P1HI,P1LO,P1PT1
	COMF	DATUM,F
	ADD	DATUM
	GOTO	EXEC
CAD2A	RDMEMA	P2HI,P2LO,P2PT1
	COMF	DATUM,F
	ADD	DATUM
	GOTO	EXEC
CAD3A	RDMEMA	P3HI,P3LO,P3PT1
	COMF	DATUM,F
	ADD	DATUM
	GOTO	EXEC
DAE	LJMP0	DAEX
DLY	LJMP0	DLYX
LDI	LJMP0	LDIX
ST0	LJMP0	ST0X
ST1	LJMP0	ST1X
ST2	LJMP0	ST2X
ST3	LJMP0	ST3X
ST1A	LJMP0	ST1AX
ST2A	LJMP0	ST2AX
ST3A	LJMP0	ST3AX
ANI	LJMP0	ANIX
ORI	LJMP0	ORIX
XRI	LJMP0	XRIX
DAI	LJMP0	DAIX
CAD0	LJMP0	CAD0X
CAD1	LJMP0	CAD1X
CAD2	LJMP0	CAD2X
CAD3	LJMP0	CAD3X
CAI	LJMP0	CAIX

; Opcode support
P0PT1	RDROM1	P0HI,P0LO
P1PT1	RDROM1	P1HI,P1LO
P2PT1	RDROM1	P2HI,P2LO
P3PT1	RDROM1	P3HI,P3LO
EAPT1	RDROM1	EAHI,EALO
JMP0	GEAPC	P0HI,P0LO
JMP1	GEAPC	P1HI,P1LO
JMP2	GEAPC	P2HI,P2LO
JMP3	GEAPC	P3HI,P3LO
NOJMP	INCPC
	GOTO	EXEC
RDA1	RDMEMA	P1HI,P1LO,P1PT1
	RETURN
RDA2	RDMEMA	P2HI,P2LO,P2PT1
	RETURN
RDA3	RDMEMA	P3HI,P3LO,P3PT1
	RETURN
DADD	DAD	DATUM
	RETURN
ILL	LJMP0	RESET
TXCHAR	LJMP0	TXX
RXCHAR	LJMP0	RXX
CHKBRK	LJMP0	CHKX

; Pages 2,3 - NIBL ROM table
	ORG	H'1000'

; ROM fudges:
;
;	Loc	Prev	New	Reason
;
;	0950	F7	FF	Allows STAT assignments to set F3 (IE)
;	028E	06	22	   / These are CSA instructions for
;	030C	06	22	  /   the purpose of testing for break.
;	0B1F	06	22	 /   To free up SB, a pseudo instruction
;	0B74	06	22	/     provides this status.
;
;	Serial I/O routines have been reduced to pseudo instructions.
;
; New pseudo instructions:
;
;	20	Send serial character - char expected in ACC
;	21	Receive serial character - returns char in ACC and EXT
;	22	Check for break - sets ACC=0(break) or ACC=20(no break)

NIBL	DT	H'08', H'C4', H'10', H'36', H'C4', H'1C', H'32', H'C4'
	DT	H'11', H'35', H'C4', H'20', H'31', H'C4', H'FF', H'C9'
	DT	H'00', H'C9', H'01', H'C4', H'0D', H'C9', H'FF', H'C4'
	DT	H'02', H'CA', H'F6', H'31', H'C4', H'20', H'35', H'B9'
	DT	H'02', H'01', H'C1', H'80', H'E4', H'0D', H'98', H'02'
	DT	H'BA', H'F6', H'C4', H'20', H'35', H'C4', H'FF', H'C9'
	DT	H'00', H'C9', H'01', H'C4', H'0D', H'C9', H'FF', H'35'
	DT	H'02', H'F4', H'10', H'E4', H'80', H'98', H'04', H'E4'
	DT	H'80', H'90', H'E9', H'C4', H'00', H'CA', H'F4', H'CA'
	DT	H'F5', H'C4', H'58', H'CA', H'FB', H'C4', H'0C', H'CA'
	DT	H'FA', H'C4', H'00', H'CA', H'EA', H'01', H'C4', H'00'
	DT	H'CA', H'80', H'AA', H'EA', H'01', H'C4', H'34', H'60'
	DT	H'9C', H'F4', H'C4', H'50', H'CA', H'FD', H'C4', H'7A'
	DT	H'CA', H'FF', H'C4', H'6A', H'CA', H'FC', H'C4', H'A6'
	DT	H'CA', H'F9', H'C4', H'8A', H'CA', H'FE', H'C2', H'FB'
	DT	H'33', H'C2', H'FA', H'37', H'C7', H'01', H'01', H'C7'
	DT	H'01', H'33', H'CA', H'FB', H'40', H'D4', H'0F', H'DC'
	DT	H'00', H'37', H'CA', H'FA', H'40', H'D4', H'F0', H'E4'
	DT	H'20', H'98', H'2F', H'E4', H'A0', H'98', H'07', H'E4'
	DT	H'C0', H'98', H'E1', H'3F', H'90', H'D8', H'C2', H'F9'
	DT	H'E4', H'D6', H'9C', H'04', H'C4', H'0A', H'90', H'60'
	DT	H'E4', H'D6', H'33', H'CA', H'EA', H'C4', H'10', H'37'
	DT	H'01', H'C2', H'FB', H'CF', H'01', H'C2', H'FA', H'CF'
	DT	H'01', H'C2', H'EA', H'33', H'CA', H'F9', H'40', H'37'
	DT	H'90', H'BA', H'CA', H'E7', H'C5', H'01', H'E4', H'20'
	DT	H'98', H'FA', H'C5', H'FF', H'C2', H'FA', H'37', H'CA'
	DT	H'EC', H'C2', H'FB', H'33', H'CA', H'ED', H'C7', H'01'
	DT	H'01', H'BA', H'E7', H'40', H'D4', H'7F', H'E5', H'01'
	DT	H'9C', H'07', H'40', H'94', H'F1', H'90', H'95', H'90'
	DT	H'8D', H'C2', H'E7', H'01', H'C5', H'80', H'C2', H'ED'
	DT	H'33', H'C2', H'EC', H'37', H'90', H'CA', H'C4', H'10'
	DT	H'37', H'C2', H'F9', H'33', H'C7', H'FF', H'01', H'C7'
	DT	H'FF', H'33', H'CA', H'F9', H'40', H'37', H'90', H'B8'
	DT	H'90', H'41', H'C2', H'FC', H'E4', H'7A', H'98', H'1C'
	DT	H'AA', H'FC', H'AA', H'FC', H'33', H'C4', H'10', H'37'
	DT	H'C2', H'F4', H'98', H'0A', H'35', H'CB', H'FF', H'35'
	DT	H'31', H'CB', H'FE', H'31', H'90', H'C1', H'C4', H'FF'
	DT	H'CB', H'FF', H'90', H'BB', H'C4', H'0A', H'90', H'1B'
	DT	H'C5', H'01', H'E4', H'20', H'98', H'FA', H'E4', H'2D'
	DT	H'98', H'04', H'E4', H'37', H'9C', H'01', H'3F', H'C4'
	DT	H'04', H'90', H'08', H'C2', H'FC', H'E4', H'6A', H'9C'
	DT	H'04', H'C4', H'09', H'90', H'40', H'BA', H'FC', H'BA'
	DT	H'FC', H'33', H'C4', H'10', H'37', H'C3', H'01', H'94'
	DT	H'06', H'C4', H'00', H'CA', H'F4', H'90', H'88', H'35'
	DT	H'C3', H'00', H'31', H'C4', H'01', H'CA', H'F4', H'90'
	DT	H'F4', H'C2', H'F2', H'94', H'04', H'C4', H'08', H'90'
	DT	H'1C', H'C4', H'01', H'CA', H'F4', H'3F', H'C4', H'0F'
	DT	H'37', H'C4', H'C1', H'33', H'C5', H'01', H'E4', H'22'
	DT	H'98', H'DB', H'E4', H'2F', H'98', H'05', H'E4', H'0D'
	DT	H'3F', H'90', H'EB', H'C4', H'07', H'90', H'35', H'C4'
	DT	H'10', H'37', H'AA', H'FD', H'AA', H'FD', H'33', H'C4'
	DT	H'0A', H'CB', H'FE', H'C4', H'00', H'CB', H'FF', H'C4'
	DT	H'05', H'CA', H'E7', H'C4', H'FF', H'CB', H'05', H'C3'
	DT	H'FD', H'94', H'13', H'C4', H'2D', H'CB', H'04', H'C4'
	DT	H'00', H'03', H'FB', H'FC', H'CB', H'FC', H'C4', H'00'
	DT	H'FB', H'FD', H'CB', H'FD', H'90', H'9F', H'C4', H'20'
	DT	H'CB', H'04', H'90', H'99', H'90', H'57', H'AA', H'FD'
	DT	H'AA', H'FD', H'31', H'C4', H'10', H'35', H'AA', H'E7'
	DT	H'01', H'C1', H'01', H'DC', H'30', H'C9', H'80', H'C1'
	DT	H'FD', H'D9', H'FC', H'98', H'0A', H'C4', H'0F', H'CA'
	DT	H'FA', H'C4', H'33', H'CA', H'FB', H'90', H'DB', H'C4'
	DT	H'0F', H'37', H'C4', H'C1', H'33', H'C2', H'F5', H'9C'
	DT	H'06', H'C1', H'04', H'3F', H'C2', H'E7', H'01', H'C5'
	DT	H'80', H'C1', H'00', H'3F', H'C5', H'FF', H'94', H'FB'
	DT	H'C4', H'50', H'CA', H'FD', H'C2', H'F5', H'9C', H'BA'
	DT	H'C4', H'20', H'3F', H'90', H'B5', H'C4', H'0F', H'37'
	DT	H'C4', H'C1', H'33', H'C4', H'0D', H'3F', H'C4', H'0A'
	DT	H'3F', H'90', H'A7', H'C4', H'05', H'CA', H'EB', H'C2'
	DT	H'EB', H'CA', H'EA', H'C4', H'0F', H'37', H'C4', H'C1'
	DT	H'33', H'C4', H'0D', H'3F', H'C4', H'0A', H'3F', H'C4'
	DT	H'0F', H'35', H'C4', H'3B', H'31', H'BA', H'EB', H'98'
	DT	H'06', H'C5', H'01', H'94', H'FC', H'90', H'F6', H'C5'
	DT	H'01', H'3F', H'C1', H'FF', H'94', H'F9', H'C2', H'EA'
	DT	H'E4', H'0E', H'98', H'0D', H'C4', H'0F', H'35', H'C4'
	DT	H'3B', H'31', H'C5', H'01', H'3F', H'C1', H'FF', H'94'
	DT	H'F9', H'C2', H'F4', H'98', H'4D', H'C4', H'20', H'3F'
	DT	H'C4', H'41', H'3F', H'C4', H'54', H'3F', H'C4', H'10'
	DT	H'37', H'AA', H'FD', H'AA', H'FD', H'33', H'C2', H'F7'
	DT	H'CB', H'FF', H'C2', H'F8', H'CB', H'FE', H'C4', H'31'
	DT	H'CA', H'FB', H'C4', H'0E', H'CA', H'FA', H'90', H'99'
	DT	H'C4', H'0E', H'90', H'99', H'C2', H'F4', H'98', H'22'
	DT	H'C1', H'00', H'D4', H'80', H'9C', H'1C', H'22', H'D4'
	DT	H'20', H'98', H'ED', H'C1', H'FF', H'E4', H'0D', H'9C'
	DT	H'08', H'C5', H'01', H'CA', H'F7', H'C5', H'02', H'CA'
	DT	H'F8', H'C4', H'0C', H'CA', H'FA', H'C4', H'86', H'CA'
	DT	H'FB', H'3F', H'C4', H'00', H'CA', H'F4', H'C4', H'50'
	DT	H'CA', H'FD', H'C4', H'1C', H'CA', H'FB', H'C4', H'0C'
	DT	H'CA', H'FA', H'C4', H'A6', H'CA', H'F9', H'90', H'BE'
	DT	H'AA', H'F4', H'C2', H'E9', H'35', H'C2', H'E8', H'31'
	DT	H'C4', H'6A', H'CA', H'FC', H'C4', H'8A', H'CA', H'FE'
	DT	H'C4', H'7A', H'CA', H'FF', H'3F', H'90', H'A7', H'90'
	DT	H'A9', H'C1', H'00', H'E4', H'80', H'94', H'18', H'C4'
	DT	H'10', H'37', H'AA', H'FD', H'AA', H'FD', H'33', H'C5'
	DT	H'01', H'CB', H'FF', H'C5', H'01', H'CB', H'FE', H'C5'
	DT	H'01', H'C4', H'01', H'CA', H'F5', H'90', H'DE', H'C4'
	DT	H'00', H'CA', H'F5', H'C4', H'02', H'37', H'C4', H'83'
	DT	H'33', H'3F', H'90', H'D1', H'90', H'D1', H'C4', H'0F'
	DT	H'37', H'C4', H'C1', H'33', H'22', H'D4', H'20', H'98'
	DT	H'E6', H'C5', H'01', H'E4', H'0D', H'98', H'05', H'E4'
	DT	H'0D', H'3F', H'90', H'F0', H'C4', H'0D', H'3F', H'C4'
	DT	H'0A', H'3F', H'02', H'C4', H'4B', H'CA', H'FB', H'C4'
	DT	H'0C', H'CA', H'FA', H'90', H'AC', H'C4', H'10', H'37'
	DT	H'BA', H'FD', H'BA', H'FD', H'33', H'02', H'C3', H'FE'
	DT	H'F3', H'00', H'CB', H'FE', H'C3', H'FF', H'F3', H'01'
	DT	H'CB', H'FF', H'90', H'BE', H'C4', H'10', H'37', H'BA'
	DT	H'FD', H'BA', H'FD', H'33', H'03', H'C3', H'FE', H'FB'
	DT	H'00', H'CB', H'FE', H'C3', H'FF', H'FB', H'01', H'CB'
	DT	H'FF', H'90', H'A7', H'C4', H'10', H'37', H'C2', H'FD'
	DT	H'33', H'03', H'C4', H'00', H'FB', H'FE', H'CB', H'FE'
	DT	H'C4', H'00', H'FB', H'FF', H'CB', H'FF', H'90', H'D2'
	DT	H'90', H'92', H'C4', H'10', H'37', H'C2', H'FD', H'33'
	DT	H'C3', H'FF', H'E3', H'FD', H'CA', H'EA', H'C3', H'FF'
	DT	H'94', H'0D', H'03', H'C4', H'00', H'FB', H'FE', H'CB'
	DT	H'FE', H'C4', H'00', H'FB', H'FF', H'CB', H'FF', H'C3'
	DT	H'FD', H'94', H'0D', H'03', H'C4', H'00', H'FB', H'FC'
	DT	H'CB', H'FC', H'C4', H'00', H'FB', H'FD', H'CB', H'FD'
	DT	H'C4', H'00', H'CB', H'00', H'CB', H'01', H'CB', H'02'
	DT	H'CB', H'03', H'C4', H'10', H'CA', H'EB', H'C3', H'FF'
	DT	H'1F', H'CB', H'FF', H'C3', H'FE', H'1F', H'CB', H'FE'
	DT	H'06', H'94', H'11', H'02', H'C3', H'02', H'F3', H'FC'
	DT	H'CB', H'02', H'C3', H'03', H'F3', H'FD', H'CB', H'03'
	DT	H'90', H'02', H'90', H'A4', H'02', H'C3', H'03', H'1F'
	DT	H'CB', H'03', H'C3', H'02', H'1F', H'CB', H'02', H'C3'
	DT	H'01', H'1F', H'CB', H'01', H'C3', H'00', H'1F', H'CB'
	DT	H'00', H'BA', H'EB', H'9C', H'C9', H'90', H'02', H'90'
	DT	H'85', H'C2', H'EA', H'94', H'0D', H'03', H'C4', H'00'
	DT	H'FB', H'00', H'CB', H'00', H'C4', H'00', H'FB', H'01'
	DT	H'CB', H'01', H'C3', H'00', H'CB', H'FC', H'C3', H'01'
	DT	H'CB', H'FD', H'BA', H'FD', H'BA', H'FD', H'90', H'DF'
	DT	H'C4', H'10', H'37', H'C2', H'FD', H'33', H'C3', H'FF'
	DT	H'DB', H'FE', H'9C', H'04', H'C4', H'0D', H'90', H'B2'
	DT	H'C3', H'FD', H'E3', H'FF', H'CA', H'EA', H'C3', H'FD'
	DT	H'94', H'11', H'C4', H'00', H'03', H'FB', H'FC', H'CB'
	DT	H'03', H'C4', H'00', H'FB', H'FD', H'CB', H'02', H'90'
	DT	H'0A', H'90', H'B4', H'C3', H'FD', H'CB', H'02', H'C3'
	DT	H'FC', H'CB', H'03', H'C3', H'FF', H'94', H'0D', H'C4'
	DT	H'00', H'03', H'FB', H'FE', H'CB', H'FE', H'C4', H'00'
	DT	H'FB', H'FF', H'CB', H'FF', H'C4', H'00', H'CB', H'01'
	DT	H'CB', H'00', H'CA', H'EB', H'CB', H'FD', H'CB', H'FC'
	DT	H'02', H'C3', H'FC', H'F3', H'FC', H'CB', H'FC', H'C3'
	DT	H'FD', H'F3', H'FD', H'CB', H'FD', H'02', H'C3', H'03'
	DT	H'F3', H'03', H'CB', H'03', H'C3', H'02', H'F3', H'02'
	DT	H'CB', H'02', H'C3', H'01', H'F3', H'01', H'CB', H'01'
	DT	H'C3', H'00', H'F3', H'00', H'CB', H'00', H'03', H'C3'
	DT	H'01', H'FB', H'FE', H'CB', H'01', H'C3', H'00', H'FB'
	DT	H'FF', H'CB', H'00', H'94', H'11', H'02', H'C3', H'01'
	DT	H'F3', H'FE', H'CB', H'01', H'C3', H'00', H'F3', H'FF'
	DT	H'CB', H'00', H'90', H'08', H'90', H'93', H'C3', H'FC'
	DT	H'DC', H'01', H'CB', H'FC', H'AA', H'EB', H'E4', H'10'
	DT	H'9C', H'AE', H'C2', H'EA', H'94', H'0D', H'C4', H'00'
	DT	H'03', H'FB', H'FC', H'CB', H'FC', H'C4', H'00', H'FB'
	DT	H'FD', H'CB', H'FD', H'BA', H'FD', H'BA', H'FD', H'90'
	DT	H'DB', H'C4', H'10', H'37', H'C2', H'FD', H'33', H'C7'
	DT	H'FD', H'01', H'C3', H'01', H'CA', H'80', H'02', H'40'
	DT	H'F4', H'01', H'01', H'C3', H'02', H'CA', H'80', H'33'
	DT	H'CA', H'FD', H'C4', H'00', H'37', H'C4', H'75', H'33'
	DT	H'3F', H'C5', H'01', H'E4', H'20', H'98', H'FA', H'C1'
	DT	H'FF', H'03', H'FC', H'5B', H'94', H'05', H'03', H'FC'
	DT	H'E6', H'94', H'12', H'C5', H'FF', H'C2', H'FB', H'33'
	DT	H'C2', H'FA', H'37', H'C3', H'00', H'CA', H'FA', H'C3'
	DT	H'01', H'CA', H'FB', H'90', H'D5', H'01', H'C1', H'00'
	DT	H'03', H'FC', H'5B', H'94', H'05', H'03', H'FC', H'E6'
	DT	H'94', H'E1', H'C4', H'10', H'37', H'AA', H'FD', H'33'
	DT	H'02', H'40', H'70', H'CB', H'FF', H'C4', H'02', H'02'
	DT	H'F2', H'FB', H'CA', H'FB', H'C4', H'00', H'F2', H'FA'
	DT	H'CA', H'FA', H'90', H'AE', H'C4', H'10', H'37', H'AA'
	DT	H'FD', H'33', H'C3', H'FE', H'01', H'C2', H'80', H'CB'
	DT	H'FE', H'02', H'40', H'F4', H'01', H'01', H'C2', H'80'
	DT	H'CB', H'FF', H'90', H'96', H'C4', H'01', H'90', H'12'
	DT	H'C4', H'02', H'90', H'0E', H'C4', H'03', H'90', H'0A'
	DT	H'C4', H'04', H'90', H'06', H'C4', H'05', H'90', H'02'
	DT	H'C4', H'06', H'CA', H'EB', H'C4', H'10', H'37', H'BA'
	DT	H'FD', H'BA', H'FD', H'33', H'03', H'C3', H'FE', H'FB'
	DT	H'00', H'CA', H'EF', H'C3', H'FF', H'FB', H'01', H'CA'
	DT	H'EE', H'E3', H'FF', H'01', H'C3', H'FF', H'E3', H'01'
	DT	H'50', H'E2', H'EE', H'CA', H'EA', H'C2', H'EE', H'DA'
	DT	H'EF', H'98', H'02', H'C4', H'80', H'E4', H'80', H'01'
	DT	H'BA', H'EB', H'9C', H'05', H'40', H'90', H'2B', H'90'
	DT	H'B1', H'BA', H'EB', H'9C', H'05', H'40', H'E4', H'80'
	DT	H'90', H'20', H'BA', H'EB', H'9C', H'04', H'C2', H'EA'
	DT	H'90', H'18', H'BA', H'EB', H'9C', H'05', H'40', H'DA'
	DT	H'EA', H'90', H'0F', H'BA', H'EB', H'9C', H'07', H'40'
	DT	H'DA', H'EA', H'E4', H'80', H'90', H'04', H'C2', H'EA'
	DT	H'E4', H'80', H'94', H'04', H'C4', H'01', H'90', H'02'
	DT	H'C4', H'00', H'CB', H'FE', H'C4', H'00', H'CB', H'FF'
	DT	H'C4', H'00', H'37', H'C4', H'F5', H'33', H'3F', H'90'
	DT	H'BE', H'C2', H'EF', H'DA', H'EE', H'98', H'02', H'90'
	DT	H'B6', H'C5', H'01', H'E4', H'0D', H'9C', H'FA', H'C4'
	DT	H'02', H'37', H'C4', H'83', H'33', H'3F', H'90', H'A7'
	DT	H'C4', H'01', H'90', H'06', H'C4', H'02', H'90', H'02'
	DT	H'C4', H'03', H'CA', H'EB', H'C4', H'10', H'37', H'BA'
	DT	H'FD', H'BA', H'FD', H'33', H'BA', H'EB', H'9C', H'0E'
	DT	H'C3', H'01', H'D3', H'FF', H'CB', H'FF', H'C3', H'00'
	DT	H'D3', H'FE', H'CB', H'FE', H'90', H'D8', H'BA', H'EB'
	DT	H'9C', H'0E', H'C3', H'01', H'DB', H'FF', H'CB', H'FF'
	DT	H'C3', H'00', H'DB', H'FE', H'CB', H'FE', H'90', H'C6'
	DT	H'C7', H'01', H'E4', H'FF', H'CB', H'FF', H'C7', H'01'
	DT	H'E4', H'FF', H'CB', H'FF', H'33', H'CA', H'FD', H'90'
	DT	H'B5', H'C2', H'F1', H'31', H'CA', H'F1', H'C2', H'F0'
	DT	H'35', H'CA', H'F0', H'3F', H'C2', H'F4', H'98', H'01'
	DT	H'3F', H'C4', H'03', H'CA', H'EB', H'C4', H'02', H'37'
	DT	H'C4', H'1E', H'33', H'3F', H'AA', H'FD', H'AA', H'FD'
	DT	H'33', H'C4', H'10', H'37', H'C4', H'00', H'CB', H'FF'
	DT	H'CB', H'FE', H'CA', H'EB', H'C5', H'01', H'E4', H'20'
	DT	H'98', H'FA', H'C5', H'FF', H'C1', H'00', H'03', H'FC'
	DT	H'3A', H'94', H'09', H'03', H'FC', H'F6', H'94', H'13'
	DT	H'90', H'32', H'90', H'BB', H'03', H'FC', H'0D', H'94'
	DT	H'2B', H'03', H'FC', H'FA', H'94', H'02', H'90', H'24'
	DT	H'02', H'F4', H'0A', H'01', H'C4', H'04', H'CA', H'EA'
	DT	H'CA', H'EB', H'C3', H'FE', H'02', H'F3', H'FE', H'CB'
	DT	H'FE', H'C3', H'FF', H'F3', H'FF', H'CB', H'FF', H'BA'
	DT	H'EA', H'9C', H'EF', H'C3', H'FE', H'58', H'CB', H'FE'
	DT	H'C5', H'01', H'90', H'C0', H'C2', H'EB', H'9C', H'87'
	DT	H'C4', H'05', H'90', H'97', H'C5', H'01', H'E4', H'20'
	DT	H'98', H'FA', H'C5', H'FF', H'03', H'FC', H'3A', H'94'
	DT	H'05', H'03', H'FC', H'F6', H'94', H'21', H'C2', H'FB'
	DT	H'33', H'C2', H'FA', H'37', H'C3', H'00', H'CA', H'FA'
	DT	H'C3', H'01', H'CA', H'FB', H'90', H'A4', H'C4', H'02'
	DT	H'02', H'F2', H'FB', H'CA', H'FB', H'C4', H'00', H'F2'
	DT	H'FA', H'CA', H'FA', H'90', H'95', H'90', H'CB', H'01'
	DT	H'C4', H'10', H'37', H'AA', H'FD', H'AA', H'FD', H'33'
	DT	H'C4', H'00', H'CB', H'FF', H'40', H'CB', H'FE', H'C5'
	DT	H'01', H'C1', H'00', H'03', H'FC', H'3A', H'94', H'D6'
	DT	H'03', H'FC', H'F6', H'94', H'02', H'90', H'CF', H'01'
	DT	H'C3', H'FF', H'CB', H'01', H'C3', H'FE', H'CB', H'00'
	DT	H'C4', H'02', H'CA', H'EA', H'02', H'C3', H'FE', H'F3'
	DT	H'FE', H'CB', H'FE', H'C3', H'FF', H'F3', H'FF', H'CB'
	DT	H'FF', H'D4', H'80', H'9C', H'34', H'BA', H'EA', H'9C'
	DT	H'EB', H'02', H'C3', H'FE', H'F3', H'00', H'CB', H'FE'
	DT	H'C3', H'FF', H'F3', H'01', H'CB', H'FF', H'D4', H'80'
	DT	H'9C', H'1F', H'02', H'C3', H'FE', H'F3', H'FE', H'CB'
	DT	H'FE', H'C3', H'FF', H'F3', H'FF', H'CB', H'FF', H'D4'
	DT	H'80', H'9C', H'0E', H'02', H'40', H'F3', H'FE', H'CB'
	DT	H'FE', H'C4', H'00', H'F3', H'FF', H'CB', H'FF', H'94'
	DT	H'9E', H'C4', H'06', H'90', H'88', H'90', H'84', H'C4'
	DT	H'10', H'35', H'C4', H'D6', H'31', H'C4', H'00', H'CA'
	DT	H'E7', H'C4', H'0F', H'37', H'C4', H'C1', H'33', H'C2'
	DT	H'F4', H'98', H'08', H'C4', H'3F', H'3F', H'C4', H'20'
	DT	H'3F', H'90', H'03', H'C4', H'3E', H'3F', H'C4', H'0F'
	DT	H'37', H'C4', H'76', H'33', H'3F', H'C4', H'C1', H'33'
	DT	H'40', H'98', H'F3', H'E4', H'0A', H'98', H'EF', H'40'
	DT	H'E4', H'0D', H'98', H'50', H'40', H'E4', H'5F', H'98'
	DT	H'41', H'40', H'E4', H'08', H'98', H'36', H'40', H'E4'
	DT	H'15', H'98', H'0F', H'40', H'E4', H'03', H'9C', H'1A'
	DT	H'C4', H'5E', H'3F', H'C4', H'43', H'3F', H'C4', H'0E'
	DT	H'90', H'A9', H'C4', H'5E', H'3F', H'C4', H'55', H'3F'
	DT	H'C4', H'0D', H'3F', H'C4', H'0A', H'3F', H'90', H'9F'
	DT	H'90', H'9B', H'40', H'CD', H'01', H'AA', H'E7', H'E4'
	DT	H'48', H'9C', H'B3', H'C4', H'0D', H'01', H'40', H'3F'
	DT	H'90', H'12', H'90', H'87', H'C4', H'20', H'3F', H'C4'
	DT	H'08', H'3F', H'C2', H'E7', H'98', H'A0', H'BA', H'E7'
	DT	H'C5', H'FF', H'90', H'9A', H'40', H'CD', H'01', H'C4'
	DT	H'0A', H'3F', H'C4', H'10', H'35', H'C4', H'D6', H'31'
	DT	H'90', H'CE', H'C4', H'10', H'37', H'C2', H'FD', H'33'
	DT	H'C3', H'FF', H'35', H'01', H'C3', H'FE', H'31', H'CA'
	DT	H'EF', H'C1', H'00', H'CB', H'FE', H'C4', H'00', H'CB'
	DT	H'FF', H'C2', H'EF', H'31', H'40', H'35', H'90', H'B0'
	DT	H'C4', H'10', H'37', H'C2', H'FD', H'33', H'C7', H'FE'
	DT	H'01', H'C7', H'FF', H'CA', H'EA', H'C7', H'FF', H'33'
	DT	H'CA', H'FD', H'C2', H'EA', H'37', H'40', H'CB', H'00'
	DT	H'90', H'C6', H'90', H'A6', H'C4', H'10', H'37', H'C2'
	DT	H'FD', H'33', H'C3', H'01', H'CA', H'F7', H'C3', H'00'
	DT	H'CA', H'F8', H'C2', H'F1', H'33', H'C2', H'F0', H'37'
	DT	H'C4', H'04', H'CA', H'E7', H'C7', H'01', H'E4', H'0D'
	DT	H'98', H'04', H'AA', H'E7', H'90', H'F6', H'C2', H'E7'
	DT	H'E4', H'04', H'9C', H'02', H'CA', H'E7', H'C2', H'E7'
	DT	H'01', H'C2', H'F2', H'94', H'06', H'D4', H'7F', H'CA'
	DT	H'F2', H'90', H'18', H'C5', H'03', H'40', H'02', H'F4'
	DT	H'FC', H'01', H'C5', H'01', H'E4', H'0D', H'98', H'0B'
	DT	H'40', H'02', H'F4', H'FF', H'01', H'90', H'F3', H'90'
	DT	H'AF', H'90', H'AF', H'40', H'DA', H'E7', H'98', H'F7'
	DT	H'C4', H'7A', H'CA', H'FF', H'C4', H'6A', H'CA', H'FC'
	DT	H'C4', H'8A', H'CA', H'FE', H'40', H'98', H'60', H'94'
	DT	H'10', H'C1', H'00', H'C9', H'80', H'C5', H'01', H'94'
	DT	H'F8', H'C1', H'00', H'94', H'F4', H'C9', H'80', H'90'
	DT	H'4E', H'C1', H'FE', H'CA', H'EA', H'C4', H'FF', H'C9'
	DT	H'FE', H'C4', H'50', H'C9', H'FF', H'C5', H'01', H'94'
	DT	H'FC', H'C1', H'00', H'94', H'F8', H'35', H'CA', H'EE'
	DT	H'35', H'31', H'CA', H'EF', H'31', H'C2', H'EF', H'02'
	DT	H'70', H'C4', H'00', H'F2', H'EE', H'E2', H'EE', H'D4'
	DT	H'F0', H'98', H'03', H'C4', H'00', H'01', H'C4', H'FF'
	DT	H'C9', H'80', H'C5', H'FF', H'94', H'FA', H'C1', H'01'
	DT	H'E4', H'50', H'98', H'04', H'C1', H'00', H'90', H'F0'
	DT	H'C2', H'EA', H'C9', H'00', H'C4', H'0D', H'C9', H'01'
	DT	H'40', H'9C', H'04', H'C4', H'02', H'90', H'8A', H'C2'
	DT	H'E7', H'98', H'84', H'C2', H'F1', H'31', H'C2', H'F0'
	DT	H'35', H'C2', H'F3', H'33', H'C2', H'F2', H'37', H'C2'
	DT	H'F7', H'CF', H'01', H'C2', H'F8', H'CF', H'01', H'C2'
	DT	H'E7', H'CF', H'01', H'C5', H'01', H'CF', H'01', H'E4'
	DT	H'0D', H'9C', H'F8', H'90', H'DC', H'C4', H'00', H'37'
	DT	H'C4', H'75', H'33', H'3F', H'90', H'CF', H'BA', H'FD'
	DT	H'BA', H'FD', H'33', H'C4', H'10', H'37', H'C3', H'00'
	DT	H'CA', H'EF', H'C3', H'01', H'CA', H'EE', H'90', H'E5'
	DT	H'C2', H'FF', H'01', H'40', H'E4', H'7A', H'9C', H'04'
	DT	H'C4', H'0F', H'90', H'E0', H'C2', H'EF', H'DA', H'EE'
	DT	H'98', H'06', H'BA', H'FF', H'BA', H'FF', H'90', H'CD'
	DT	H'40', H'33', H'C4', H'10', H'37', H'C3', H'FF', H'35'
	DT	H'C3', H'FE', H'31', H'90', H'C0', H'C2', H'EF', H'D4'
	DT	H'FF', H'07', H'90', H'B9', H'90', H'BE', H'C4', H'10'
	DT	H'37', H'AA', H'FD', H'AA', H'FD', H'33', H'06', H'CB'
	DT	H'FE', H'C4', H'00', H'CB', H'FF', H'90', H'EB', H'C2'
	DT	H'EE', H'37', H'C2', H'EF', H'33', H'C7', H'FF', H'3F'
	DT	H'C4', H'10', H'36', H'C4', H'1C', H'32', H'90', H'DA'
	DT	H'C2', H'FF', H'E4', H'8A', H'9C', H'04', H'C4', H'0A'
	DT	H'90', H'D2', H'AA', H'FF', H'AA', H'FF', H'33', H'C4'
	DT	H'10', H'37', H'35', H'CB', H'FF', H'35', H'31', H'CB'
	DT	H'FE', H'31', H'90', H'BE', H'C2', H'E9', H'37', H'C2'
	DT	H'E8', H'33', H'C3', H'00', H'94', H'02', H'90', H'07'
	DT	H'C3', H'02', H'01', H'C7', H'80', H'90', H'F3', H'C7'
	DT	H'02', H'AA', H'FD', H'AA', H'FD', H'33', H'01', H'C4'
	DT	H'10', H'37', H'CB', H'FF', H'40', H'CB', H'FE', H'90'
	DT	H'D9', H'C5', H'01', H'E4', H'0D', H'9C', H'FA', H'3F'
	DT	H'C2', H'FD', H'33', H'C4', H'10', H'37', H'C3', H'03'
	DT	H'CB', H'FE', H'C3', H'02', H'CB', H'FF', H'90', H'C2'
	DT	H'90', H'AE', H'C4', H'08', H'CA', H'EB', H'C2', H'E5'
	DT	H'01', H'C2', H'E4', H'CA', H'E9', H'C2', H'E5', H'02'
	DT	H'70', H'01', H'C2', H'E4', H'02', H'F2', H'E9', H'CA'
	DT	H'E4', H'BA', H'EB', H'9C', H'F0', H'40', H'02', H'F4'
	DT	H'07', H'01', H'C2', H'E4', H'02', H'F4', H'07', H'1E'
	DT	H'CA', H'E4', H'AA', H'E6', H'98', H'03', H'40', H'CA'
	DT	H'E5', H'C2', H'FD', H'33', H'C4', H'10', H'37', H'C4'
	DT	H'01', H'CB', H'00', H'C4', H'00', H'CB', H'01', H'C3'
	DT	H'FE', H'CB', H'02', H'C3', H'FF', H'CB', H'03', H'C3'
	DT	H'FC', H'CB', H'04', H'C3', H'FD', H'CB', H'05', H'C2'
	DT	H'E4', H'CB', H'FE', H'C2', H'E5', H'E4', H'FF', H'D4'
	DT	H'7F', H'CB', H'FF', H'C7', H'06', H'33', H'CA', H'FD'
	DT	H'90', H'9C', H'90', H'9C', H'AA', H'FD', H'AA', H'FD'
	DT	H'33', H'C4', H'10', H'37', H'C4', H'00', H'CB', H'FF'
	DT	H'C4', H'01', H'CB', H'FE', H'90', H'EA', H'C2', H'FE'
	DT	H'E4', H'A6', H'9C', H'04', H'C4', H'0A', H'90', H'E2'
	DT	H'E4', H'A6', H'31', H'CA', H'F1', H'C4', H'10', H'35'
	DT	H'CA', H'F0', H'C2', H'FD', H'33', H'C4', H'10', H'37'
	DT	H'C3', H'F9', H'CD', H'01', H'C3', H'FC', H'CD', H'01'
	DT	H'C3', H'FD', H'CD', H'01', H'C3', H'FE', H'CD', H'01'
	DT	H'C3', H'FF', H'CD', H'01', H'C2', H'F1', H'CD', H'01'
	DT	H'C2', H'F0', H'CD', H'01', H'35', H'C2', H'F1', H'31'
	DT	H'CA', H'FE', H'C7', H'FC', H'33', H'CA', H'FD', H'90'
	DT	H'A7', H'C2', H'FE', H'E4', H'8A', H'9C', H'04', H'C4'
	DT	H'0B', H'90', H'BB', H'E4', H'8A', H'31', H'CA', H'F1'
	DT	H'C4', H'10', H'35', H'CA', H'F0', H'C2', H'FD', H'33'
	DT	H'C4', H'10', H'37', H'C7', H'FF', H'E1', H'F9', H'98'
	DT	H'04', H'C4', H'0C', H'90', H'A1', H'E1', H'F9', H'01'
	DT	H'C2', H'80', H'02', H'F1', H'FC', H'CA', H'80', H'CB'
	DT	H'00', H'C6', H'01', H'C2', H'80', H'F1', H'FD', H'CA'
	DT	H'80', H'CB', H'01', H'C6', H'FF', H'C1', H'FA', H'CB'
	DT	H'02', H'C1', H'FB', H'CB', H'03', H'C1', H'FD', H'94'
	DT	H'10', H'C4', H'04', H'CA', H'EB', H'C7', H'01', H'E4'
	DT	H'FF', H'CB', H'FF', H'BA', H'EB', H'9C', H'F6', H'90'
	DT	H'02', H'C7', H'04', H'33', H'CA', H'FD', H'C2', H'F1'
	DT	H'31', H'C2', H'F0', H'35', H'90', H'99', H'C2', H'EF'
	DT	H'98', H'08', H'C2', H'FE', H'02', H'F4', H'F9', H'CA'
	DT	H'FE', H'3F', H'C2', H'FE', H'33', H'C4', H'10', H'37'
	DT	H'C3', H'FF', H'35', H'C3', H'FE', H'31', H'90', H'E4'
	DT	H'90', H'A1', H'C2', H'EE', H'35', H'C2', H'EF', H'31'
	DT	H'C4', H'0F', H'37', H'C4', H'C1', H'33', H'C5', H'01'
	DT	H'E4', H'0D', H'98', H'D0', H'E4', H'0D', H'3F', H'22'
	DT	H'D4', H'20', H'9C', H'F2', H'90', H'C6', H'C2', H'EE'
	DT	H'37', H'C2', H'EF', H'33', H'C5', H'01', H'CF', H'01'
	DT	H'E4', H'0D', H'9C', H'F8', H'90', H'B6', H'C2', H'EF'
	DT	H'33', H'C2', H'EE', H'37', H'C5', H'01', H'E4', H'22'
	DT	H'98', H'0E', H'E4', H'2F', H'9C', H'04', H'C4', H'07'
	DT	H'90', H'BE', H'E4', H'0D', H'CF', H'01', H'90', H'EC'
	DT	H'C4', H'0D', H'CB', H'00', H'90', H'DE', H'C2', H'FD'
	DT	H'33', H'C4', H'10', H'37', H'C7', H'FF', H'35', H'C7'
	DT	H'FF', H'31', H'C7', H'FF', H'01', H'C7', H'FF', H'33'
	DT	H'CA', H'FD', H'40', H'37', H'C5', H'01', H'CF', H'01'
	DT	H'E4', H'0D', H'98', H'C0', H'22', H'D4', H'20', H'9C'
	DT	H'F3', H'90', H'B9', H'AA', H'FD', H'AA', H'FD', H'33'
	DT	H'C4', H'10', H'37', H'C2', H'F6', H'CB', H'FE', H'C4'
	DT	H'00', H'CB', H'FF', H'90', H'A7', H'C2', H'EF', H'D4'
	DT	H'07', H'9C', H'02', H'C4', H'01', H'CA', H'F6', H'3F'
	DT	H'C2', H'F6', H'E4', H'01', H'9C', H'09', H'C4', H'11'
	DT	H'CA', H'E9', H'C4', H'20', H'CA', H'E8', H'3F', H'E4'
	DT	H'01', H'01', H'C4', H'04', H'CA', H'EB', H'40', H'02'
	DT	H'70', H'01', H'BA', H'EB', H'9C', H'F8', H'40', H'CA'
	DT	H'E9', H'C4', H'02', H'CA', H'E8', H'3F', H'C2', H'E9'
	DT	H'35', H'C2', H'E8', H'31', H'3F', H'35', H'01', H'40'
	DT	H'35', H'40', H'1C', H'1C', H'1C', H'1C', H'CA', H'F6'
	DT	H'3F', H'C2', H'E9', H'35', H'C2', H'E8', H'31', H'C4'
	DT	H'0D', H'C9', H'FF', H'C4', H'FF', H'C9', H'00', H'C9'
	DT	H'01', H'3F', H'C2', H'E9', H'35', H'C2', H'E8', H'31'
	DT	H'C1', H'00', H'E4', H'FF', H'94', H'12', H'03', H'C1'
	DT	H'01', H'FA', H'EF', H'C1', H'00', H'FA', H'EE', H'94'
	DT	H'07', H'C1', H'02', H'01', H'C5', H'80', H'90', H'E8'
	DT	H'31', H'CA', H'F3', H'31', H'35', H'CA', H'F2', H'35'
	DT	H'C2', H'EF', H'E1', H'01', H'9C', H'07', H'C2', H'EE'
	DT	H'E1', H'00', H'9C', H'01', H'3F', H'C2', H'F2', H'DC'
	DT	H'80', H'CA', H'F2', H'3F', H'02', H'0C', H'07', H'56'
	DT	H'2C', H'25', H'8D', H'4C', H'1E', H'06', H'AB', H'0C'
	DT	H'35', H'0B', H'97', H'06', H'30', H'09', H'15', H'0B'
	DT	H'E1', H'08', H'23', H'4C', H'1E', H'2C', H'51', H'4C'
	DT	H'49', H'53', H'D4', H'0B', H'97', H'06', H'AB', H'0C'
	DT	H'47', H'09', H'15', H'0B', H'E1', H'4C', H'49', H'0B'
	DT	H'BD', H'02', H'D8', H'8F', H'2F', H'03', H'05', H'4C'
	DT	H'1C', H'2C', H'60', H'52', H'55', H'CE', H'01', H'2F'
	DT	H'0B', H'97', H'0B', H'BD', H'02', H'BF', H'02', H'83'
	DT	H'2C', H'6D', H'43', H'4C', H'45', H'41', H'D2', H'01'
	DT	H'2F', H'00', H'50', H'02', H'83', H'2C', H'86', H'4E'
	DT	H'45', H'D7', H'06', H'AB', H'0C', H'78', H'4C', H'7A'
	DT	H'0A', H'33', H'01', H'2F', H'09', H'15', H'0B', H'8C'
	DT	H'0B', H'97', H'0B', H'D0', H'02', H'83', H'2C', H'8B'
	DT	H'4C', H'45', H'D4', H'04', H'E0', H'0C', H'9A', H'2E'
	DT	H'2F', H'BD', H'8E', H'35', H'04', H'C0', H'01', H'2F'
	DT	H'02', H'83', H'2C', H'AA', H'C0', H'8E', H'AC', H'2E'
	DT	H'2F', H'BD', H'8E', H'35', H'08', H'07', H'01', H'2F'
	DT	H'02', H'83', H'2C', H'BC', H'49', H'C6', H'8E', H'35'
	DT	H'2C', H'B6', H'54', H'48', H'45', H'CE', H'09', H'15'
	DT	H'05', H'D0', H'4C', H'86', H'2C', H'D1', H'55', H'4E'
	DT	H'54', H'49', H'CC', H'06', H'3B', H'8E', H'35', H'01'
	DT	H'2F', H'09', H'15', H'09', H'27', H'0B', H'C4', H'02'
	DT	H'83', H'2C', H'DD', H'44', H'CF', H'06', H'3B', H'01'
	DT	H'2F', H'09', H'77', H'02', H'83', H'2D', H'00', H'47'
	DT	H'CF', H'2C', H'EB', H'54', H'CF', H'8E', H'35', H'01'
	DT	H'2F', H'4C', H'F6', H'2E', H'2F', H'53', H'55', H'C2'
	DT	H'8E', H'35', H'01', H'2F', H'01', H'09', H'0B', H'97'
	DT	H'09', H'15', H'0B', H'E1', H'01', H'68', H'02', H'83'
	DT	H'2D', H'10', H'52', H'45', H'54', H'55', H'52', H'CE'
	DT	H'01', H'2F', H'01', H'42', H'0B', H'C4', H'02', H'83'
	DT	H'2D', H'2A', H'4E', H'45', H'58', H'D4', H'06', H'3B'
	DT	H'04', H'E0', H'0E', H'2F', H'01', H'2F', H'0A', H'88'
	DT	H'8E', H'5F', H'09', H'15', H'0A', H'ED', H'0B', H'C4'
	DT	H'02', H'83', H'2D', H'54', H'46', H'4F', H'D2', H'06'
	DT	H'3B', H'04', H'E0', H'0E', H'2F', H'2E', H'2F', H'BD'
	DT	H'8E', H'35', H'2E', H'2F', H'54', H'CF', H'8E', H'35'
	DT	H'2D', H'4A', H'53', H'54', H'45', H'D0', H'8E', H'35'
	DT	H'4D', H'4C', H'0A', H'33', H'01', H'2F', H'0A', H'45'
	DT	H'04', H'C0', H'02', H'83', H'2D', H'67', H'53', H'54'
	DT	H'41', H'D4', H'2E', H'2F', H'BD', H'8E', H'35', H'09'
	DT	H'15', H'09', H'4C', H'01', H'2F', H'02', H'83', H'2D'
	DT	H'7E', H'50', H'41', H'47', H'C5', H'2E', H'2F', H'BD'
	DT	H'8E', H'35', H'01', H'2F', H'09', H'15', H'0B', H'8C'
	DT	H'0B', H'97', H'0B', H'BD', H'02', H'83', H'2D', H'9E'
	DT	H'A4', H'8E', H'AC', H'2E', H'2F', H'BD', H'2D', H'8F'
	DT	H'A2', H'09', H'15', H'0B', H'35', H'4D', H'9A', H'2E'
	DT	H'2F', H'A4', H'8E', H'AC', H'06', H'30', H'0B', H'55'
	DT	H'06', H'30', H'01', H'2F', H'02', H'83', H'2D', H'D1'
	DT	H'50', H'D2', H'2D', H'A7', H'49', H'4E', H'D4', H'2D'
	DT	H'AE', H'A2', H'01', H'75', H'4D', H'C1', H'2D', H'BD'
	DT	H'A4', H'8E', H'AC', H'06', H'30', H'09', H'15', H'0B'
	DT	H'09', H'06', H'30', H'4D', H'C1', H'8E', H'35', H'8F'
	DT	H'2F', H'2D', H'C6', H'AC', H'4D', H'A7', H'2D', H'CB'
	DT	H'BB', H'4D', H'CD', H'02', H'0C', H'01', H'2F', H'02'
	DT	H'83', H'2E', H'09', H'49', H'4E', H'50', H'55', H'D4'
	DT	H'06', H'3B', H'04', H'E0', H'0D', H'F6', H'06', H'30'
	DT	H'07', H'56', H'8E', H'35', H'04', H'C0', H'06', H'30'
	DT	H'2E', H'05', H'AC', H'04', H'E0', H'0E', H'2F', H'06'
	DT	H'30', H'2E', H'2F', H'AC', H'4D', H'E2', H'2E', H'2F'
	DT	H'A4', H'8E', H'AC', H'06', H'30', H'07', H'56', H'09'
	DT	H'15', H'0B', H'25', H'06', H'30', H'01', H'2F', H'02'
	DT	H'83', H'2E', H'12', H'45', H'4E', H'C4', H'01', H'2F'
	DT	H'02', H'7F', H'2E', H'26', H'4C', H'49', H'4E', H'CB'
	DT	H'8E', H'35', H'01', H'2F', H'06', H'30', H'09', H'15'
	DT	H'09', H'66', H'06', H'30', H'02', H'83', H'2E', H'2F'
	DT	H'52', H'45', H'CD', H'09', H'B8', H'02', H'83', H'02'
	DT	H'1A', H'8F', H'2F', H'02', H'A9', H'8E', H'61', H'2E'
	DT	H'3E', H'BD', H'8E', H'61', H'05', H'43', H'2E', H'53'
	DT	H'BC', H'2E', H'48', H'BD', H'8E', H'61', H'05', H'4F'
	DT	H'2E', H'4F', H'BE', H'8E', H'61', H'05', H'47', H'8E'
	DT	H'61', H'05', H'4B', H'2E', H'8B', H'BE', H'2E', H'5D'
	DT	H'BD', H'8E', H'61', H'05', H'57', H'8E', H'61', H'05'
	DT	H'53', H'2E', H'6A', H'AD', H'8E', H'8D', H'03', H'5A'
	DT	H'4E', H'6F', H'2E', H'6D', H'AB', H'8E', H'8D', H'2E'
	DT	H'78', H'AB', H'8E', H'8D', H'03', H'2C', H'4E', H'6F'
	DT	H'2E', H'81', H'AD', H'8E', H'8D', H'03', H'43', H'4E'
	DT	H'6F', H'2E', H'8B', H'4F', H'D2', H'8E', H'8D', H'05'
	DT	H'EB', H'4E', H'6F', H'00', H'F5', H'8E', H'AC', H'2E'
	DT	H'98', H'AA', H'8E', H'AC', H'03', H'71', H'4E', H'8F'
	DT	H'2E', H'A1', H'AF', H'8E', H'AC', H'04', H'07', H'4E'
	DT	H'8F', H'2E', H'8B', H'41', H'4E', H'C4', H'8E', H'AC'
	DT	H'05', H'E7', H'4E', H'8F', H'04', H'E0', H'0E', H'B4'
	DT	H'05', H'2B', H'00', H'F5', H'06', H'AB', H'0E', H'BA'
	DT	H'00', H'F5', H'2E', H'C1', H'A3', H'06', H'4B', H'00'
	DT	H'F5', H'2E', H'CB', H'A8', H'8E', H'35', H'2E', H'2F'
	DT	H'A9', H'00', H'F5', H'2E', H'D4', H'C0', H'8E', H'AC'
	DT	H'07', H'E9', H'00', H'F5', H'2E', H'DF', H'4E', H'4F'
	DT	H'D4', H'8E', H'AC', H'05', H'EF', H'00', H'F5', H'2E'
	DT	H'E9', H'53', H'54', H'41', H'D4', H'09', H'55', H'00'
	DT	H'F5', H'2E', H'F4', H'54', H'4F', H'D0', H'0B', H'97'
	DT	H'09', H'93', H'00', H'F5', H'2F', H'01', H'4D', H'4F'
	DT	H'C4', H'8F', H'20', H'04', H'07', H'09', H'BF', H'00'
	DT	H'F5', H'2F', H'16', H'52', H'4E', H'C4', H'8F', H'20'
	DT	H'09', H'D1', H'03', H'43', H'03', H'2C', H'04', H'07'
	DT	H'09', H'BF', H'03', H'2C', H'00', H'F5', H'2E', H'2F'
	DT	H'50', H'41', H'47', H'C5', H'0B', H'7A', H'00', H'F5'
	DT	H'2E', H'2F', H'A8', H'8E', H'35', H'2E', H'2F', H'AC'
	DT	H'8E', H'35', H'2E', H'2F', H'A9', H'00', H'F5', H'06'
	DT	H'30', H'01', H'8E', H'04', H'07', H'01', H'C5', H'06'
	DT	H'30', H'00', H'F5', H'20', H'45', H'52', H'52', H'4F'
	DT	H'D2', H'41', H'52', H'45', H'C1', H'53', H'54', H'4D'
	DT	H'D4', H'43', H'48', H'41', H'D2', H'53', H'4E', H'54'
	DT	H'D8', H'56', H'41', H'4C', H'D5', H'45', H'4E', H'44'
	DT	H'A2', H'4E', H'4F', H'47', H'CF', H'52', H'54', H'52'
	DT	H'CE', H'4E', H'45', H'53', H'D4', H'4E', H'45', H'58'
	DT	H'D4', H'46', H'4F', H'D2', H'44', H'49', H'56', H'B0'
	DT	H'42', H'52', H'CB', H'55', H'4E', H'54', H'CC', H'21'
	DT	H'3F', H'90', H'FC', H'00', H'00', H'00', H'00', H'00'
	DT	H'00', H'00', H'00', H'00', H'00', H'00', H'00', H'00'
	DT	H'00', H'00', H'00', H'00', H'00', H'00', H'00', H'00'
	DT	H'00', H'00', H'00', H'00', H'00', H'00', H'00', H'00'
	DT	H'00', H'00', H'00', H'00', H'00', H'00', H'00', H'00'
	DT	H'00', H'00', H'00', H'00', H'00', H'00', H'00', H'00'
	DT	H'00', H'00', H'00', H'00', H'00', H'00', H'00', H'00'
	DT	H'00', H'00', H'00', H'00', H'00', H'00', H'00', H'00'
	DT	H'00', H'00', H'00', H'00', H'00', H'00', H'00', H'00'
	DT	H'00', H'00', H'20', H'3F', H'90', H'FC', H'00', H'00'
	DT	H'00', H'00', H'00', H'00', H'00', H'00', H'00', H'00'
	DT	H'00', H'00', H'00', H'00', H'00', H'00', H'00', H'00'
	DT	H'00', H'00', H'00', H'00', H'00', H'00', H'00', H'00'
	DT	H'00', H'00', H'00', H'00', H'00', H'00', H'00', H'00'
	DT	H'00', H'00', H'00', H'00', H'00', H'00', H'00', H'00'
	DT	H'00', H'00', H'00', H'00', H'00', H'00', H'00', H'00'
	DT	H'00', H'00', H'00', H'00', H'00', H'00', H'00', H'00'

	END
